//	GamePlay.cpp - Main gameplay state.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "GamePlay.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "Images.h"
#include "Stats.h"
#include "Random.h"
#include "BattleMapGenerators.h"
#include "OverworldMapGenerator.h"
#include "MiscHelper.h"

#include <sstream>
#include <cassert>

using namespace std;

//------------------------------------------------------------------------------
uint32_t GamePlay::backTexture = 0;
uint32_t GamePlay::mouseImage = 0;
uint32_t GamePlay::titleImage = 0;
uint32_t GamePlay::animalImages[6] = {0, 0, 0, 0, 0, 0};
uint32_t GamePlay::humanImages[3] = {0, 0, 0};
uint32_t GamePlay::foodImage = 0;
uint32_t GamePlay::statBoostImage = 0;
uint32_t GamePlay::backpackImage = 0;
uint32_t GamePlay::selectorImage = 0;
uint32_t GamePlay::buttonImage = 0;
uint32_t GamePlay::healthMeterTopImage = 0;
uint32_t GamePlay::healthMeterBottomImage = 0;
uint32_t GamePlay::healthMeterBarImage = 0;
uint32_t GamePlay::heartImage = 0;
uint32_t GamePlay::equipSlotImage = 0;
uint32_t GamePlay::messageBoxImage = 0;
uint32_t GamePlay::gameOverImage = 0;
uint32_t GamePlay::battleImage = 0;
uint32_t GamePlay::win1Image = 0;
uint32_t GamePlay::win2Image = 0;
uint32_t GamePlay::font = 0;

//------------------------------------------------------------------------------
GamePlay::GamePlay(Application *app):
GameState(app),
selectingTeam(true),
numSelected(0),
overworld(false),
inventory(false),
turnCounter(1),
numWardens(0),
overworldSelectedAnimal(-1),
battleSelectedAnimal(-1),
status(InPlay),
battleDisplay(0.0f)
{
	int i;

	for(i=0;i<6;++i)
		selectedAnimals[i] = false;
}

//------------------------------------------------------------------------------
GamePlay::~GamePlay()
{
	
}

//------------------------------------------------------------------------------
void GamePlay::draw(Drawer *d)
{
	int i;
	float tempf;
	const TwoFloats tileBounds(128, 128);

	if(!backTexture)
	{
		ImageLoader imLoad;
		ImageData data;

		data = imLoad.loadImage(Images::backtexture_jpg, Images::backtexture_jpgSize);
		backTexture = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::mousepointer_png, Images::mousepointer_pngSize);
		mouseImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::title_png, Images::title_pngSize);
		titleImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::animalelephant_png, Images::animalelephant_pngSize);
		animalImages[0] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::animallion_png, Images::animallion_pngSize);
		animalImages[1] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::animalhippo_png, Images::animalhippo_pngSize);
		animalImages[2] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::animalgiraffe_png, Images::animalgiraffe_pngSize);
		animalImages[3] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::animalowl_png, Images::animalowl_pngSize);
		animalImages[4] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::animalwallaby_png, Images::animalwallaby_pngSize);
		animalImages[5] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::humanparkwarden_png, Images::humanparkwarden_pngSize);
		humanImages[0] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::humanpolice_png, Images::humanpolice_pngSize);
		humanImages[1] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::humansoldier_png, Images::humansoldier_pngSize);
		humanImages[2] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::invcake_png, Images::invcake_pngSize);
		foodImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::backpack_png, Images::backpack_pngSize);
		backpackImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::invhat_png, Images::invhat_pngSize);
		statBoostImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::tileselector_png, Images::tileselector_pngSize);
		selectorImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::button_png, Images::button_pngSize);
		buttonImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::healthmetertop_png, Images::healthmetertop_pngSize);
		healthMeterTopImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::healthmeterbottom_png, Images::healthmeterbottom_pngSize);
		healthMeterBottomImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::healthmeterbar_png, Images::healthmeterbar_pngSize);
		healthMeterBarImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::heart_png, Images::heart_pngSize);
		heartImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::equipslot_png, Images::equipslot_pngSize);
		equipSlotImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::messagebox_png, Images::messagebox_pngSize);
		messageBoxImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::gameover_png, Images::gameover_pngSize);
		gameOverImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::battle_png, Images::battle_pngSize);
		battleImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::win1_png, Images::win1_pngSize);
		win1Image = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::win2_png, Images::win2_pngSize);
		win2Image = d->loadImage(data.data, data.width, data.height);

		OverworldMap::loadTiles(d);
		BattleMap::loadTiles(d);

		font = d->loadFont("Chunk.ttf", 28.0f);
	}

	d->drawImage(backTexture, 0, 0);
	d->drawImage(backTexture, 1024, 0);

	if(selectingTeam)
		drawTeamSelection(d);
	else if(inventory)
		drawInventory(d);
	else if(overworld)
		drawOverworld(d);
	else //Battle screen.
		drawBattle(d);

	if(messageBox.isVisible())
		messageBox.draw(d, messageBoxImage, buttonImage, font);

	switch(status)
	{
		case GameOver:
			d->drawImage(gameOverImage, 640-512, 360-512);
			break;
		case Win1:
			d->drawImage(win1Image, 640-512, 360-512);
			break;
		case Win2:
			d->drawImage(win2Image, 640-512, 360-512);
			break;
	}

	//Draw mouse cursor.
	d->drawImage(mouseImage, mousePos.x-8, mousePos.y-8);
}

//------------------------------------------------------------------------------
void GamePlay::keyDown(const string& action, const SDL_keysym& keyPressed)
{
	if(action == "Escape")
		quit();
}

//------------------------------------------------------------------------------
void GamePlay::mouseDown(int button, int x, int y)
{

}

//------------------------------------------------------------------------------
void GamePlay::mouseUp(int button, int x, int y)
{
	int i, j;
	const TwoFloats tileBounds(128, 128);

	if(status != InPlay)
	{
		quit();
		return;
	}

	mousePos.x = (float)x;
	mousePos.y = (float)y;

	if(messageBox.isVisible())
	{
		messageBox.mouseUp(TwoFloats((float)x, (float)y));
		return;
	}

	if(selectingTeam)
	{
		i = 0;
		if(mousePos.isWithin(TwoFloats(122, 720-219-128), tileBounds))
		{
			selectedAnimals[i] = !selectedAnimals[i];
			if(selectedAnimals[i])
			{
				if(numSelected == 3)
					selectedAnimals[i] = false;
				else
					++numSelected;
			}
			else
				--numSelected;
		}
		++i;
		if(mousePos.isWithin(TwoFloats(303, 720-219-128), tileBounds))
		{
			selectedAnimals[i] = !selectedAnimals[i];
			if(selectedAnimals[i])
			{
				if(numSelected == 3)
					selectedAnimals[i] = false;
				else
					++numSelected;
			}
			else
				--numSelected;
		}
		++i;
		if(mousePos.isWithin(TwoFloats(485, 720-219-128), tileBounds))
		{
			selectedAnimals[i] = !selectedAnimals[i];
			if(selectedAnimals[i])
			{
				if(numSelected == 3)
					selectedAnimals[i] = false;
				else
					++numSelected;
			}
			else
				--numSelected;
		}
		++i;
		if(mousePos.isWithin(TwoFloats(666, 720-219-128), tileBounds))
		{
			selectedAnimals[i] = !selectedAnimals[i];
			if(selectedAnimals[i])
			{
				if(numSelected == 3)
					selectedAnimals[i] = false;
				else
					++numSelected;
			}
			else
				--numSelected;
		}
		++i;
		if(mousePos.isWithin(TwoFloats(848, 720-219-128), tileBounds))
		{
			selectedAnimals[i] = !selectedAnimals[i];
			if(selectedAnimals[i])
			{
				if(numSelected == 3)
					selectedAnimals[i] = false;
				else
					++numSelected;
			}
			else
				--numSelected;
		}
		++i;
		if(mousePos.isWithin(TwoFloats(1029, 720-219-128), tileBounds))
		{
			selectedAnimals[i] = !selectedAnimals[i];
			if(selectedAnimals[i])
			{
				if(numSelected == 3)
					selectedAnimals[i] = false;
				else
					++numSelected;
			}
			else
				--numSelected;
		}

		//Start the game if the player's ready.
		if(mousePos.isWithin(TwoFloats(1019, 611), TwoFloats(220, 83)) &&
		   (numSelected > 0))
		{
			selectingTeam = false;
			overworld = true;

			//Generate the overworld map.
			createMaps();

			//Setup the player's animals.
			j = 0;
			for(i=0;i<6;++i)
			{
				if(selectedAnimals[i])
				{
					animals.push_back(createAnimal(i));

					//Position the animals around the safari park.
					switch(j)
					{
						case 0:
							animals[j]->setPosition(TwoFloats(3, 4));
							break;
						case 1:
							animals[j]->setPosition(TwoFloats(4, 3));
							break;
						case 2:
							animals[j]->setPosition(TwoFloats(5, 4));
							break;
					}
					++j;
				}
			}
		}
	}
	else if(inventory)
	{
		if(mousePos.isWithin(TwoFloats(1019, 611), TwoFloats(220, 83)))
			inventory = false;
	}
	else if(overworld)
	{
		/*if(mousePos.isWithin(TwoFloats(8, 648), TwoFloats(64, 64)))
		{
			inventory = true;
			return;
		}*/

		if(overworldSelectedAnimal > -1)
		{
			TwoFloats potentials[4];
			bool adjacentTile = false;
			shared_ptr<AnimalActorOver> selectedAnimal = animals[overworldSelectedAnimal];
			const TwoFloats& animalPos = selectedAnimal->getPosition();

			potentials[0].x = animalPos.x-1; potentials[0].y = animalPos.y;
			potentials[1].x = animalPos.x; potentials[1].y = animalPos.y-1;
			potentials[2].x = animalPos.x+1; potentials[2].y = animalPos.y;
			potentials[3].x = animalPos.x; potentials[3].y = animalPos.y+1;

			for(i=0;i<4;++i)
			{
				if((potentials[i].x < 0.0f) || (potentials[i].x >= (float)OverworldMap::MapWidth) ||
				   (potentials[i].y < 0.0f) || (potentials[i].y >= (float)OverworldMap::MapHeight))
				{
					continue;
				}

				if(mousePos.isWithin(128.0f * potentials[i], tileBounds))
				{
					adjacentTile = true;

					switch(overworldMap[((int)potentials[i].y * OverworldMap::MapWidth)+(int)potentials[i].x])
					{
						case OverworldMap::Forest:
							if(selectedAnimal->getStats().canForest)
								selectedAnimal->setPosition(potentials[i]);
							else
								messageBox.display("This animal is too big to hide in a forest.");
							break;
						case OverworldMap::Lake:
							if(selectedAnimal->getStats().canSwim)
								selectedAnimal->setPosition(potentials[i]);
							else
								messageBox.display("This animal can't swim.");
							break;
						case OverworldMap::SafariPark:
							animals.erase(animals.begin()+overworldSelectedAnimal);
							overworldSelectedAnimal = -1;
							messageBox.display("The animal returns to the safari park.");
							break;
						default:
							selectedAnimal->setPosition(potentials[i]);
					}

					/*for(j=0;j<overworldItems.size();++j)
					{
						if(overworldItems[j]->position == selectedAnimal->getPosition())
						{
							if(!overworldItems[j]->food)
							{
								if(selectedAnimal->getAnimal(0)->addEquippedItem(overworldItems[j]))
								{
									stringstream tempstr;
								
									tempstr << "You discovered a " << overworldItems[j]->name << "! You've equipped it.";
									messageBox.display(tempstr.str());
									overworldItems.erase(overworldItems.begin()+j);
									break;
								}
								else if(selectedAnimal->getAnimal(1)->addEquippedItem(overworldItems[j]))
								{
									stringstream tempstr;
								
									tempstr << "You discovered a " << overworldItems[j]->name << "! You've equipped it.";
									messageBox.display(tempstr.str());
									overworldItems.erase(overworldItems.begin()+j);
									break;
								}
							}
						}
					}*/

					incrementTurnCounter();

					break;
				}

				if(adjacentTile)
					break;
			}

			if(!adjacentTile)
			{
				overworldSelectedAnimal = -1;

				for(i=0;i<animals.size();++i)
				{
					if(mousePos.isWithin(128.0f * animals[i]->getPosition(), tileBounds))
					{
						overworldSelectedAnimal = i;
						break;
					}
				}
			}
		}
		else
		{
			for(i=0;i<animals.size();++i)
			{
				if(mousePos.isWithin(128.0f * animals[i]->getPosition(), tileBounds))
				{
					overworldSelectedAnimal = i;
					break;
				}
			}
		}
	}
	else //Battle.
	{
		/*if(mousePos.isWithin(TwoFloats(8, 648), TwoFloats(64, 64)))
		{
			inventory = true;
			return;
		}*/

		if(battleSelectedAnimal > -1)
		{
			TwoFloats potentials[4];
			bool adjacentTile = false;
			shared_ptr<AnimalActor> battleSelected = battleAnimals[battleSelectedAnimal];
			const TwoFloats& animalPos = battleSelected->getPosition();

			potentials[0].x = animalPos.x-1; potentials[0].y = animalPos.y;
			potentials[1].x = animalPos.x; potentials[1].y = animalPos.y-1;
			potentials[2].x = animalPos.x+1; potentials[2].y = animalPos.y;
			potentials[3].x = animalPos.x; potentials[3].y = animalPos.y+1;

			for(i=0;i<4;++i)
			{
				if((potentials[i].x < 0.0f) || (potentials[i].x >= (float)BattleMap::MapWidth) ||
				   (potentials[i].y < 0.0f) || (potentials[i].y >= (float)BattleMap::MapHeight))
				{
					continue;
				}

				if(mousePos.isWithin(128.0f * potentials[i], tileBounds))
				{
					adjacentTile = true;

					switch(battleMaps[battleIndex][((int)potentials[i].y * BattleMap::MapWidth)+(int)potentials[i].x])
					{
						case BattleMap::Tree:
							messageBox.display("You can't walk through trees.");
							break;
						case BattleMap::Lake:
							if(battleSelected->getStats().canSwim)
								battleAnimalMove(battleSelected, potentials[i]);
							else
								messageBox.display("This animal can't swim.");
							break;
						case BattleMap::Skyscraper:
							messageBox.display("You can't walk through skyscrapers.");
							break;
						default:
							//battleSelected->setPosition(potentials[i]);
							battleAnimalMove(battleSelected, potentials[i]);
					}

					//Move Humans, potentially attack Animals, check battle over.
					if(battleHumans.size() > 0)
						battleTurn();

					break;
				}

				if(adjacentTile)
					break;
			}

			if(!adjacentTile)
			{
				battleSelectedAnimal = -1;

				for(i=0;i<battleAnimals.size();++i)
				{
					if(mousePos.isWithin(128.0f * battleAnimals[i]->getPosition(), tileBounds))
					{
						battleSelectedAnimal = i;
						break;
					}
				}
			}
		}
		else
		{
			for(i=0;i<battleAnimals.size();++i)
			{
				if(mousePos.isWithin(128.0f * battleAnimals[i]->getPosition(), tileBounds))
				{
					battleSelectedAnimal = i;
					break;
				}
			}
		}
	}
}

//------------------------------------------------------------------------------
void GamePlay::mouseMove(int x, int y)
{
	mousePos.x = (float)x;
	mousePos.y = (float)y;

	if(messageBox.isVisible())
		messageBox.mouseMove(TwoFloats((float)x, (float)y));
}

//------------------------------------------------------------------------------
void GamePlay::drawTeamSelection(Drawer *d)
{
	int i;
	float tempf;
	const TwoFloats tileBounds(128, 128);

	d->fillRect(0, 0, 1280, 720, 32, 192, 32, 128);

	d->drawImage(titleImage, 0, 0);
	d->drawText("You're breaking out of this dead-end safari park.",
				font,
				306,
				720-429-28-34,
				1024);
	d->drawText("Pick the 3 animals you're going to escape with:",
				font,
				321,
				720-429-28,
				1024);
	d->drawText("Evade your captors for as long as possible",
				font,
				32,
				720-128,
				1024);
	d->drawText("and one day you'll be able to live free and",
				font,
				32,
				720-128+34,
				1024);
	d->drawText("wild in the countryside.",
				font,
				32,
				720-128+34+34,
				1024);

	//Draw Elephant.
	if(mousePos.isWithin(TwoFloats(122, 720-219-128), tileBounds) || selectedAnimals[Elephant])
		d->drawImage(selectorImage, 122, 720-219-128);
	d->drawImage(animalImages[Elephant], 122, 720-219-128);
	d->drawText("Elephant", font, 124, 720-174-28, 192);

	//Draw Lion.
	if(mousePos.isWithin(TwoFloats(303, 720-219-128), tileBounds) || selectedAnimals[Lion])
		d->drawImage(selectorImage, 303, 720-219-128);
	d->drawImage(animalImages[Lion], 303, 720-219-128);
	d->drawText("Lion", font, 338, 720-174-28, 192);

	//Draw Hippo.
	if(mousePos.isWithin(TwoFloats(485, 720-219-128), tileBounds) || selectedAnimals[Hippo])
		d->drawImage(selectorImage, 485, 720-219-128);
	d->drawImage(animalImages[Hippo], 485, 720-219-128);
	d->drawText("Hippo", font, 508, 720-174-28, 192);

	//Draw Giraffe.
	if(mousePos.isWithin(TwoFloats(666, 720-219-128), tileBounds) || selectedAnimals[Giraffe])
		d->drawImage(selectorImage, 666, 720-219-128);
	d->drawImage(animalImages[Giraffe], 666, 720-219-128);
	d->drawText("Giraffe", font, 683, 720-174-28, 192);

	//Draw Owl.
	if(mousePos.isWithin(TwoFloats(848, 720-219-128), tileBounds) || selectedAnimals[Owl])
		d->drawImage(selectorImage, 848, 720-219-128);
	d->drawImage(animalImages[Owl], 848, 720-219-128);
	d->drawText("Owl", font, 885, 720-174-28, 192);

	//Draw Wallaby.
	if(mousePos.isWithin(TwoFloats(1029, 720-219-128), tileBounds) || selectedAnimals[Wallaby])
		d->drawImage(selectorImage, 1029, 720-219-128);
	d->drawImage(animalImages[Wallaby], 1029, 720-219-128);
	d->drawText("Wallaby", font, 1035, 720-174-28, 192);

	//Draw Escape! button.
	d->drawImage(buttonImage, (1280-256-16), (720-192));
	if(mousePos.isWithin(TwoFloats(1019, 611), TwoFloats(220, 83)))
		d->drawText("Escape!", font, (1280-256-16)+28+8, (720-192)+87, 182, 1.5f, 0.66f, 0.66f, 1.0f);
	else
		d->drawText("Escape!", font, (1280-256-16)+28+8, (720-192)+87, 182, 1.5f);

	//Draw stats if necessary.
	if(mousePos.isWithin(TwoFloats(122, 720-219-128), tileBounds))
		statsDisplay.draw(d, font, "Elephant", GlobalStats::AnimalStats[Elephant], mousePos+48);
	if(mousePos.isWithin(TwoFloats(303, 720-219-128), tileBounds))
		statsDisplay.draw(d, font, "Lion", GlobalStats::AnimalStats[Lion], mousePos+48);
	if(mousePos.isWithin(TwoFloats(485, 720-219-128), tileBounds))
		statsDisplay.draw(d, font, "Hippo", GlobalStats::AnimalStats[Hippo], mousePos+48);
	if(mousePos.isWithin(TwoFloats(666, 720-219-128), tileBounds))
		statsDisplay.draw(d, font, "Giraffe", GlobalStats::AnimalStats[Giraffe], mousePos+48);
	if(mousePos.isWithin(TwoFloats(848, 720-219-128), tileBounds))
		statsDisplay.draw(d, font, "Owl", GlobalStats::AnimalStats[Owl], mousePos+48);
	if(mousePos.isWithin(TwoFloats(1029, 720-219-128), tileBounds))
		statsDisplay.draw(d, font, "Wallaby", GlobalStats::AnimalStats[Wallaby], mousePos+48);
}

//------------------------------------------------------------------------------
void GamePlay::drawOverworld(Drawer *d)
{
	int i;
	float tempf;
	const TwoFloats tileBounds(128, 128);

	//Draw background.
	d->fillRect(0, 0, 1280, 640, 32, 192, 32, 128);
	d->fillRect(0, 640, 1280, 80, 96, 64, 8, 128);
	d->fillRect(0, 639, 1280, 2, 0, 0, 0, 128);

	//Draw map.
	OverworldMap::drawMap(d, overworldMap);

	//Draw humans.
	for(i=0;i<humans.size();++i)
		humans[i]->draw(d, 128.0f * humans[i]->getPosition(), 1.0f);

	//Draw player's animals.
	for(i=0;i<animals.size();++i)
		animals[i]->draw(d, 128.0f * animals[i]->getPosition(), 1.0f);

	//Draw selected animal.
	if(overworldSelectedAnimal > -1)
	{
		TwoFloats potentials[4];
		const TwoFloats& animalPos = animals[overworldSelectedAnimal]->getPosition();

		d->drawImage(selectorImage,
						128.0f * animalPos.x,
						128.0f * animalPos.y,
						1.0f,
						1.0f,
						1.0f,
						1.0f,
						0.5f);

		//Draw potential moves.
		potentials[0].x = animalPos.x-1; potentials[0].y = animalPos.y;
		potentials[1].x = animalPos.x; potentials[1].y = animalPos.y-1;
		potentials[2].x = animalPos.x+1; potentials[2].y = animalPos.y;
		potentials[3].x = animalPos.x; potentials[3].y = animalPos.y+1;

		for(i=0;i<4;++i)
		{
			if((potentials[i].x < 0.0f) || (potentials[i].x >= (float)OverworldMap::MapWidth) ||
				(potentials[i].y < 0.0f) || (potentials[i].y >= (float)OverworldMap::MapHeight))
			{
				continue;
			}

			if(mousePos.isWithin(128.0f * potentials[i], tileBounds))
			{
				switch(overworldMap[((int)potentials[i].y * OverworldMap::MapWidth)+(int)potentials[i].x])
				{
					case OverworldMap::Forest:
						if(animals[overworldSelectedAnimal]->getStats().canForest)
						{
							d->drawImage(selectorImage,
										128.0f * potentials[i].x,
										128.0f * potentials[i].y,
										1.0f,
										1.0f,
										0.125f,
										0.5f,
										0.125f);
						}
						else
						{
							d->drawImage(selectorImage,
											128.0f * potentials[i].x,
											128.0f * potentials[i].y,
											1.0f,
											1.0f,
											0.75f,
											0.25f,
											0.25f);
						}
						break;
					case OverworldMap::Lake:
						if(animals[overworldSelectedAnimal]->getStats().canSwim)
						{
							d->drawImage(selectorImage,
										128.0f * potentials[i].x,
										128.0f * potentials[i].y,
										1.0f,
										1.0f,
										0.125f,
										0.5f,
										0.125f);
						}
						else
						{
							d->drawImage(selectorImage,
											128.0f * potentials[i].x,
											128.0f * potentials[i].y,
											1.0f,
											1.0f,
											0.75f,
											0.25f,
											0.25f);
						}
						break;
					default:
						d->drawImage(selectorImage,
										128.0f * potentials[i].x,
										128.0f * potentials[i].y,
										1.0f,
										1.0f,
										0.125f,
										0.5f,
										0.125f);
				}

				break;
			}
		}
	}

	//Draw inventory button.
	//d->drawImage(backpackImage, 8, 648);

	//Draw player's animals in the hotbar.
	for(i=0;i<animals.size();++i)
	{
		animals[i]->draw(d, TwoFloats(80+(i * 368), (640+8)), 0.5f);
		d->drawImage(heartImage, 80+(i * 368)+64, (640+8));
		d->drawImage(healthMeterBottomImage, 80+(i * 368)+128, (640+8)-30);
		tempf = (float)animals[i]->getHealth()/(float)animals[i]->getMaxHealth();
		d->drawSubImage(healthMeterBarImage,
						80+(i * 368)+128,
						(640+8)-30,
						0.0f,
						0.0f,
						tempf, //Width.
						1.0f,
						1.0f,
						1.0f,
						0.25f,
						0.75f,
						0.25f);
		d->drawImage(healthMeterTopImage, 80+(i * 368)+128, (640+8)-30);
		//d->drawText(animals[i]->getName(), font, 8+(i * 384), 640, 128, 0.75f);
	}

	//Draw turn counter.
	{
		stringstream tempstr;

		d->drawText("Turn:", font, 1046+16+24, 652, 256, 1.5f);
		tempstr << turnCounter;
		d->drawText(tempstr.str(), font, 1170+16+24, 652, 256, 1.5f, 1.0f, 1.0f, 0.5f);
	}

	//Draw inventory tooltip if necessary.
	/*if(mousePos.isWithin(TwoFloats(8, 648), TwoFloats(64, 64)))
	{
		d->fillRect(mousePos.x, mousePos.y-24, 128, 24, 0, 0, 0, 128);
		d->drawText("Inventory", font, mousePos.x+8, mousePos.y-26, 124, 0.75f);
	}*/

	//Draw animal tooltip if necessary.
	for(i=0;i<animals.size();++i)
	{
		if(mousePos.isWithin(128.0f * animals[i]->getPosition(), tileBounds))
		{
			statsDisplay.draw(d,
								font,
								animals[i]->getName(),
								animals[i]->getStats(),
								mousePos+48);
		}
	}
	//Draw human tooltip if necessary.
	for(i=0;i<humans.size();++i)
	{
		if(mousePos.isWithin(128.0f * humans[i]->getPosition(), tileBounds))
		{
			statsDisplay.draw(d,
								font,
								humans[i]->getName(),
								humans[i]->getStats(),
								mousePos+48);
		}
	}
}

//------------------------------------------------------------------------------
void GamePlay::drawBattle(Drawer *d)
{
	int i;
	float tempf;
	const TwoFloats tileBounds(128, 128);

	//Draw background.
	d->fillRect(0, 0, 1280, 640, 32, 192, 32, 128);
	d->fillRect(0, 640, 1280, 80, 96, 64, 8, 128);
	d->fillRect(0, 639, 1280, 2, 0, 0, 0, 128);

	//Draw map.
	BattleMap::drawMap(d, battleMaps[((int)currentBattle.y*OverworldMap::MapWidth)+(int)currentBattle.x]);

	//Draw Humans.
	for(i=0;i<battleHumans.size();++i)
	{
		TwoFloats tempPos = 128.0f * battleHumans[i]->getPosition();

		battleHumans[i]->draw(d, tempPos, 1.0f);

		//Draw Animal's health.
		d->fillRect(tempPos.x,
					tempPos.y,
					128.0f*((float)battleHumans[i]->getHealth()/(float)battleHumans[i]->getStats().health),
					8,
					64,
					255,
					64,
					192);
	}

	//Draw Animals.
	for(i=0;i<battleAnimals.size();++i)
	{
		TwoFloats tempPos = 128.0f * battleAnimals[i]->getPosition();

		battleAnimals[i]->draw(d, tempPos, 1.0f);

		//Draw Animal's health.
		d->fillRect(tempPos.x,
					tempPos.y,
					128.0f*((float)battleAnimals[i]->getHealth()/(float)battleAnimals[i]->getStats().health),
					8,
					64,
					255,
					64,
					192);
	}

	//Draw any on-screen BattleTexts.
	for(i=(damageTexts.size()-1);i>=0;--i)
	{
		if(damageTexts[i]->draw(d, font))
			damageTexts.erase(damageTexts.begin()+i);
	}

	//Draw selected animal.
	if(battleSelectedAnimal > -1)
	{
		TwoFloats potentials[4];
		const TwoFloats& animalPos = battleAnimals[battleSelectedAnimal]->getPosition();

		d->drawImage(selectorImage,
						128.0f * animalPos.x,
						128.0f * animalPos.y,
						1.0f,
						1.0f,
						1.0f,
						1.0f,
						0.5f);

		//Draw potential moves.
		potentials[0].x = animalPos.x-1; potentials[0].y = animalPos.y;
		potentials[1].x = animalPos.x; potentials[1].y = animalPos.y-1;
		potentials[2].x = animalPos.x+1; potentials[2].y = animalPos.y;
		potentials[3].x = animalPos.x; potentials[3].y = animalPos.y+1;

		for(i=0;i<4;++i)
		{
			if((potentials[i].x < 0.0f) || (potentials[i].x >= (float)BattleMap::MapWidth) ||
				(potentials[i].y < 0.0f) || (potentials[i].y >= (float)BattleMap::MapHeight))
			{
				continue;
			}

			if(mousePos.isWithin(128.0f * potentials[i], tileBounds))
			{
				switch(battleMaps[battleIndex][((int)potentials[i].y * BattleMap::MapWidth)+(int)potentials[i].x])
				{
					case BattleMap::Tree:
						d->drawImage(selectorImage,
									 128.0f * potentials[i].x,
									 128.0f * potentials[i].y,
									 1.0f,
									 1.0f,
									 0.75f,
									 0.25f,
									 0.25f);
						break;
					case BattleMap::Lake:
						d->drawImage(selectorImage,
									 128.0f * potentials[i].x,
									 128.0f * potentials[i].y,
									 1.0f,
									 1.0f,
									 0.75f,
									 0.25f,
									 0.25f);
						break;
					case BattleMap::Skyscraper:
						d->drawImage(selectorImage,
									 128.0f * potentials[i].x,
									 128.0f * potentials[i].y,
									 1.0f,
									 1.0f,
									 0.75f,
									 0.25f,
									 0.25f);
						break;
					default:
						d->drawImage(selectorImage,
										128.0f * potentials[i].x,
										128.0f * potentials[i].y,
										1.0f,
										1.0f,
										0.125f,
										0.5f,
										0.125f);
				}

				break;
			}
		}
	}

	//Draw inventory button.
	//d->drawImage(backpackImage, 8, 648);

	//Draw player's animals in the hotbar.
	for(i=0;i<animals.size();++i)
	{
		animals[i]->draw(d, TwoFloats(80+(i * 368), (640+8)), 0.5f);
		d->drawImage(heartImage, 80+(i * 368)+64, (640+8));
		d->drawImage(healthMeterBottomImage, 80+(i * 368)+128, (640+8)-30);
		tempf = (float)animals[i]->getHealth()/(float)animals[i]->getMaxHealth();
		d->drawSubImage(healthMeterBarImage,
						80+(i * 368)+128,
						(640+8)-30,
						0.0f,
						0.0f,
						tempf, //Width.
						1.0f,
						1.0f,
						1.0f,
						0.25f,
						0.75f,
						0.25f);
		d->drawImage(healthMeterTopImage, 80+(i * 368)+128, (640+8)-30);
		//d->drawText(animals[i]->getName(), font, 8+(i * 384), 640, 128, 0.75f);
	}

	//Draw turn counter.
	{
		stringstream tempstr;

		d->drawText("Turn:", font, 1046+16+24, 652, 256, 1.5f);
		tempstr << turnCounter;
		d->drawText(tempstr.str(), font, 1170+16+24, 652, 256, 1.5f, 1.0f, 1.0f, 0.5f);
	}

	//Draw inventory tooltip if necessary.
	/*if(mousePos.isWithin(TwoFloats(8, 648), TwoFloats(64, 64)))
	{
		d->fillRect(mousePos.x, mousePos.y-24, 128, 24, 0, 0, 0, 128);
		d->drawText("Inventory", font, mousePos.x+8, mousePos.y-26, 124, 0.75f);
	}*/

	//Draw animal tooltip if necessary.
	for(i=0;i<battleAnimals.size();++i)
	{
		if(mousePos.isWithin(128.0f * battleAnimals[i]->getPosition(), tileBounds))
		{
			statsDisplay.draw(d,
								font,
								battleAnimals[i]->getName(),
								battleAnimals[i]->getStats(),
								mousePos+48);
		}
	}
	//Draw human tooltip if necessary.
	for(i=0;i<battleHumans.size();++i)
	{
		if(mousePos.isWithin(128.0f * battleHumans[i]->getPosition(), tileBounds))
		{
			statsDisplay.draw(d,
								font,
								battleHumans[i]->getName(),
								battleHumans[i]->getStats(),
								mousePos+48);
		}
	}

	if(battleDisplay > 0.0f)
	{
		d->drawImage(battleImage, 640-512, 360-512-64+battleDisplay, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, fakeLog(battleDisplay/64.0f));
		--battleDisplay;
	}
}

//------------------------------------------------------------------------------
void GamePlay::drawInventory(Drawer *d)
{
	int i;
	int x = 64;
	shared_ptr<AnimalActor> tempAnimal;

	//Draw background.
	d->fillRect(0, 0, 1280, 720, 128, 64, 32, 128);

	for(i=0;i<animals.size();++i)
	{
		tempAnimal = animals[i]->getAnimal(0);

		if(!tempAnimal->getDead())
		{
			const shared_ptr<Item>& equippedItem = tempAnimal->getEquippedItem();

			tempAnimal->draw(d, TwoFloats(x, 32), 1.0f);
			d->drawText("Equipment", font, x-16, 180, 160);
			d->drawImage(equipSlotImage, x+32, 224);
			if(equippedItem)
			{
				if(equippedItem->food)
					d->drawImage(foodImage, x+32, 224);
				else
					d->drawImage(statBoostImage, x+32, 224);
			}
			d->drawText("Inventory", font, x-16, 296, 160);
			d->fillRect(x+32, 340, 64, 256, 0, 0, 0, 128);

			x += 128 + 64;
		}

		tempAnimal = animals[i]->getAnimal(1);

		if(!tempAnimal->getDead())
		{
			const shared_ptr<Item>& equippedItem = tempAnimal->getEquippedItem();

			tempAnimal->draw(d, TwoFloats(x, 32), 1.0f);
			d->drawText("Equipment", font, x-16, 180, 160);
			d->drawImage(equipSlotImage, x+32, 224);
			if(equippedItem)
			{
				if(equippedItem->food)
					d->drawImage(foodImage, x+32, 224);
				else
					d->drawImage(statBoostImage, x+32, 224);
			}
			d->drawText("Inventory", font, x-16, 296, 160);
			d->fillRect(x+32, 340, 64, 256, 0, 0, 0, 128);

			x += 128 + 64;
		}
	}

	//Draw exit button.
	d->drawImage(buttonImage, (1280-256-16), (720-192));
	if(mousePos.isWithin(TwoFloats(1019, 611), TwoFloats(220, 83)))
		d->drawText("Exit", font, (1280-256-16)+28+40, (720-192)+87, 182, 1.5f, 0.66f, 0.66f, 1.0f);
	else
		d->drawText("Exit", font, (1280-256-16)+28+40, (720-192)+87, 182, 1.5f);
}

//------------------------------------------------------------------------------
void GamePlay::incrementTurnCounter()
{
	int i, j;
	int x, y;
	TwoFloats conflictTile = getConflictTile();

	++turnCounter;

	if(turnCounter >= NumWinningTurns)
	{
		status = Win2;
		humans.clear();
		return;
	}

	if((turnCounter > 2) && (numWardens < 3))
	{
		humans.push_back(shared_ptr<HumanActorOver>(new HumanActorOver("Park Warden",
																	   GlobalStats::HumanStats[ParkWarden],
																	   humanImages[ParkWarden])));
		humans.back()->setPosition(TwoFloats(4.0f, 4.0f));

		++numWardens;
	}

	//Move the Human nearest to an animal towards it.
	if((turnCounter > 4) && humans.size() && animals.size())
	{
		float tempf;
		int human = 0;
		int animal = 0;
		float dist = 100.0f;

		//Make sure we don't move anyone if the player's initiated a battle.
		if(conflictTile == TwoFloats(-1, -1))
		{
			for(i=0;i<humans.size();++i)
			{
				for(j=0;j<animals.size();++j)
				{
					//Don't count animals hiding in lakes.
					if(!isOnLake(animals[j]->getPosition()))
					{
						tempf = humans[i]->getPosition().getDistance(animals[j]->getPosition());
						if(tempf <= dist)
						{
							human = i;
							animal = j;
							dist = tempf;
						}
					}
				}
			}
			humans[human]->move(animals, overworldMap);

			conflictTile = getConflictTile();
		}

	}

	//Trigger battle.
	if(conflictTile != TwoFloats(-1, -1))
	{
		OverworldMap::Tile tempTile = overworldMap[((int)conflictTile.y*OverworldMap::MapWidth)+(int)conflictTile.x];
		overworld = false;
		battleSelectedAnimal = -1;
		currentBattle = conflictTile;
		battleIndex = ((int)currentBattle.y * OverworldMap::MapWidth) + (int)currentBattle.x;
		battleDisplay = 64.0f;

		if(!battleHumanOver->getHuman(0)->getDead())
			battleHumans.push_back(battleHumanOver->getHuman(0));
		if(!battleHumanOver->getHuman(1)->getDead())
			battleHumans.push_back(battleHumanOver->getHuman(1));
		if(!battleAnimalOver->getAnimal(0)->getDead())
			battleAnimals.push_back(battleAnimalOver->getAnimal(0));
		if(!battleAnimalOver->getAnimal(1)->getDead())
			battleAnimals.push_back(battleAnimalOver->getAnimal(1));

		//Place the Actors correctly on the BattleMap.
		for(i=0;i<battleHumans.size();++i)
		{
			x = (int)(Random::getGlobal().getRandomFloat() * ((float)BattleMap::MapWidth-0.01f));

			while((battleMaps[battleIndex][x] == BattleMap::Tree) ||
				  (battleMaps[battleIndex][x] == BattleMap::Lake))
			{
				x = (int)(Random::getGlobal().getRandomFloat() * ((float)BattleMap::MapWidth-0.01f));
			}
			battleHumans[i]->setPosition(TwoFloats(x, 0));
		}
		for(i=0;i<battleAnimals.size();++i)
		{
			x = (int)(Random::getGlobal().getRandomFloat() * ((float)BattleMap::MapWidth-0.01f));
			y = (BattleMap::MapHeight-1);

			while((battleMaps[battleIndex][(y*BattleMap::MapWidth)+x] == BattleMap::Tree) ||
				  (battleMaps[battleIndex][(y*BattleMap::MapWidth)+x] == BattleMap::Lake) ||
				  (battleMaps[battleIndex][(y*BattleMap::MapWidth)+x] == BattleMap::Skyscraper))
			{
				x = (int)(Random::getGlobal().getRandomFloat() * ((float)BattleMap::MapWidth-0.01f));
			}
			battleAnimals[i]->setPosition(TwoFloats(x, y));
		}
	}
}

//------------------------------------------------------------------------------
void GamePlay::battleTurn()
{
	int i, j;
	float tempf;
	int human = 0;
	int animal = 0;
	float dist = 100.0f;
	TwoFloats oldHumanPos;
	TwoFloats newHumanPos;
	shared_ptr<AnimalActor> victim;

	assert(battleHumans.size());

	//Get the Human nearest an Animal to move towards it.
	for(i=0;i<battleHumans.size();++i)
	{
		if(!battleHumans[i]->getDead())
		{
			for(j=0;j<battleAnimals.size();++j)
			{
				if(!battleAnimals[j]->getDead())
				{
					tempf = battleHumans[i]->getPosition().getDistance(battleAnimals[j]->getPosition());
					if(tempf <= dist)
					{
						human = i;
						animal = j;
						dist = tempf;
					}
				}
			}
		}
	}
	oldHumanPos = battleHumans[human]->getPosition();
	battleHumans[human]->move(battleAnimals, battleMaps[battleIndex]);
	newHumanPos = battleHumans[human]->getPosition();

	//Check if the human has moved onto the same tile as an Animal.
	for(i=0;i<battleAnimals.size();++i)
	{
		if(newHumanPos == battleAnimals[i]->getPosition())
		{
			victim = battleAnimals[i];
			break;
		}
	}

	//If so, attack the Animal.
	if(victim)
	{
		int attackVal = victim->attacked(battleHumans[human]);

		damageTexts.push_back(shared_ptr<DamageText>(new DamageText(attackVal, (128.0f * victim->getPosition())+40.0f)));

		if(victim->getHealth() <= 0)
		{
			int deadCount = 0;

			victim->setDead();

			//Check if the battle has been won.
			for(i=0;i<battleAnimals.size();++i)
			{
				if(battleAnimals[i]->getDead())
					++deadCount;
			}
			//If it has, quit the battle...
			if(deadCount == battleAnimals.size())
			{
				for(i=0;i<animals.size();++i)
				{
					//...And remove the associated HumanActorOver from the overworld.
					if(animals[i] == battleAnimalOver)
					{
						animals.erase(animals.begin()+i);
						break;
					}
				}
				overworld = true;
				overworldSelectedAnimal = -1;
				battleAnimals.clear();
				battleHumans.clear();

				if(animals.size() == 0)
					status = GameOver;
				else
					messageBox.display("You lost the battle. Your animals have been recaptured.");

				return;
			}
		}

		//Reset the Human's position.
		battleHumans[human]->setPosition(oldHumanPos);
	}
}

//------------------------------------------------------------------------------
void GamePlay::battleAnimalMove(shared_ptr<AnimalActor>& animal,
								const TwoFloats& pos)
{
	int i, j;
	shared_ptr<HumanActor> victim;
	vector<shared_ptr<AnimalActor> > attackers;

	for(i=0;i<battleHumans.size();++i)
	{
		if(battleHumans[i]->getPosition() == pos)
		{
			victim = battleHumans[i];
			break;
		}
	}

	if(victim)
	{
		//Find out how many Animals are adjacent to the victim.
		for(i=0;i<battleAnimals.size();++i)
		{
			if(battleAnimals[i]->getPosition().getDistance(victim->getPosition()) < 2.0f)
			{
				int attackVal = victim->attacked(battleAnimals[i]);

				if(i == 0)
					damageTexts.push_back(shared_ptr<DamageText>(new DamageText(attackVal, (128.0f * victim->getPosition())+40.0f)));
				else
					damageTexts.push_back(shared_ptr<DamageText>(new DamageText(attackVal, (128.0f * victim->getPosition())+40.0f+24.0f)));

				if(victim->getHealth() <= 0)
				{
					int deadCount = 0;

					victim->setDead();

					//Check if the battle has been won.
					for(j=0;j<battleHumans.size();++j)
					{
						if(battleHumans[j]->getDead())
							++deadCount;
					}
					//If it has, quit the battle...
					if(deadCount == battleHumans.size())
					{
						for(j=0;j<humans.size();++j)
						{
							//...And remove the associated HumanActorOver from the overworld.
							if(humans[j] == battleHumanOver)
							{
								humans.erase(humans.begin()+j);
								break;
							}
						}
						overworld = true;
						battleAnimals.clear();
						battleHumans.clear();

						if(humans.size() == 0)
							status = Win1;
						else
							messageBox.display("You won the battle. Those humans won't be hunting you down any time soon.");

						break;
					}
				}
			}
		}
	}
	else
		animal->setPosition(pos);
}

//------------------------------------------------------------------------------
void GamePlay::createMaps()
{
	int i;
	int x, y;
	int index;
	float tempf;
	BattleMapGenerator battleGen;
	OverworldMapGenerator mapGen;

	//Generate the overworld map.
	overworldMap = mapGen.generate();

	//Generate all the battle maps.
	for(y=0;y<OverworldMap::MapHeight;++y)
	{
		for(x=0;x<OverworldMap::MapWidth;++x)
		{
			index = (y*OverworldMap::MapWidth)+x;
			switch(overworldMap[index])
			{
				case OverworldMap::Grass:
					battleMaps[index] = battleGen.generateGrassMap();
					break;
				case OverworldMap::Field:
					battleMaps[index] = battleGen.generateFieldMap();
					break;
				case OverworldMap::Forest:
					battleMaps[index] = battleGen.generateForestMap();
					break;
				case OverworldMap::Town:
					battleMaps[index] = battleGen.generateTownMap();
					break;
				case OverworldMap::City:
					battleMaps[index] = battleGen.generateCityMap();
					break;
				default: //Can't fight in Lakes.
					battleMaps[index] = battleGen.generateNullMap();
			}
		}

	}

	//Add HumanActors to towns and cities.
	for(y=0;y<OverworldMap::MapHeight;++y)
	{
		for(x=0;x<OverworldMap::MapWidth;++x)
		{
			OverworldMap::Tile tile = overworldMap[(y*OverworldMap::MapWidth)+x];

			tempf = Random::getGlobal().getRandomFloat();
			if((tile == OverworldMap::Town) && (tempf > 0.5f))
			{
				humans.push_back(shared_ptr<HumanActorOver>(new HumanActorOver("Police",
																			   GlobalStats::HumanStats[Police],
																			   humanImages[Police])));
				humans.back()->setPosition(TwoFloats((float)x, (float)y));
			}
			else if(tile == OverworldMap::City)
			{
				if(tempf > 0.66f)
				{
					humans.push_back(shared_ptr<HumanActorOver>(new HumanActorOver("Soldier",
																				   GlobalStats::HumanStats[Soldier],
																				   humanImages[Soldier])));
					humans.back()->setPosition(TwoFloats((float)x, (float)y));
				}
				else if(tempf > 0.33f)
				{
					humans.push_back(shared_ptr<HumanActorOver>(new HumanActorOver("Police",
																			   GlobalStats::HumanStats[Police],
																			   humanImages[Police])));
					humans.back()->setPosition(TwoFloats((float)x, (float)y));
				}
			}
		}
	}

	//Add Items to the maps.
	for(y=0;y<OverworldMap::MapHeight;++y)
	{
		for(x=0;x<OverworldMap::MapWidth;++x)
		{
			if(Random::getGlobal().getRandomFloat() > 0.66f)
				overworldItems.push_back(shared_ptr<Item>(new Item(TwoFloats(x, y))));
		}
	}
	for(i=0;i<(OverworldMap::MapWidth*OverworldMap::MapHeight);++i)
	{
		for(y=0;y<BattleMap::MapHeight;++y)
		{
			for(x=0;x<BattleMap::MapWidth;++x)
			{
				if(Random::getGlobal().getRandomFloat() > 0.66f)
					battleItems[i].push_back(shared_ptr<Item>(new Item(TwoFloats(x, y))));
			}
		}
	}
}

//------------------------------------------------------------------------------
shared_ptr<AnimalActorOver> GamePlay::createAnimal(int index)
{
	shared_ptr<AnimalActorOver> retval;

	switch(index)
	{
		case Elephant:
			retval = shared_ptr<AnimalActorOver>(new AnimalActorOver("Elephant",
																	 GlobalStats::AnimalStats[Elephant],
																	 animalImages[index]));
			break;
		case Lion:
			retval = shared_ptr<AnimalActorOver>(new AnimalActorOver("Lion",
																	 GlobalStats::AnimalStats[Lion],
																	 animalImages[index]));
			break;
		case Hippo:
			retval = shared_ptr<AnimalActorOver>(new AnimalActorOver("Hippo",
																	 GlobalStats::AnimalStats[Hippo],
																	 animalImages[index]));
			break;
		case Giraffe:
			retval = shared_ptr<AnimalActorOver>(new AnimalActorOver("Giraffe",
																	 GlobalStats::AnimalStats[Giraffe],
																	 animalImages[index]));
			break;
		case Owl:
			retval = shared_ptr<AnimalActorOver>(new AnimalActorOver("Owl",
																	 GlobalStats::AnimalStats[Owl],
																	 animalImages[index]));
			break;
		case Wallaby:
			retval = shared_ptr<AnimalActorOver>(new AnimalActorOver("Wallaby",
																	 GlobalStats::AnimalStats[Wallaby],
																	 animalImages[index]));
			break;
	}

	return retval;
}

//------------------------------------------------------------------------------
TwoFloats GamePlay::getConflictTile()
{
	int i, j;
	TwoFloats retval(-1, -1);

	for(i=0;i<animals.size();++i)
	{
		for(j=0;j<humans.size();++j)
		{
			if(animals[i]->getPosition() == humans[j]->getPosition())
			{
				retval = animals[i]->getPosition();
				battleAnimalOver = animals[i];
				battleHumanOver = humans[j];
				break;
			}
		}
		if(retval != TwoFloats(-1, -1))
			break;
	}

	return retval;
}

//------------------------------------------------------------------------------
bool GamePlay::isOnLake(const TwoFloats& pos)
{
	return (overworldMap[((int)pos.y*OverworldMap::MapWidth)+(int)pos.x] == OverworldMap::Lake);
}
