//	GamePlay.h - Main gameplay state.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef GAMEPLAY_H_
#define GAMEPLAY_H_

#include "GameState.h"
#include "TwoFloats.h"
#include "StatsDisplay.h"
#include "OverworldMap.h"
#include "BattleMap.h"
#include "AnimalActorOver.h"
#include "HumanActorOver.h"
#include "MessageBox.h"
#include "DamageText.h"
#include "Item.h"

#include <stdint.h>
#include <memory>
#include <vector>

///	For convenience.
typedef std::array<BattleMap::Map, (OverworldMap::MapWidth*OverworldMap::MapHeight)> OverworldBattleMaps;

///	Main gameplay state.
class GamePlay : public GameState
{
  public:
	///	Constructor.
	GamePlay(Application *app);
	///	Destructor.
	~GamePlay();

	///	Draws the state.
	void draw(Drawer *d);

	///	Called when a key down event is received.
	void keyDown(const std::string& action, const SDL_keysym& keyPressed);

	///	Called when a mouse down event is received.
	void mouseDown(int button, int x, int y);
	///	Called when a mouse up event is received.
	void mouseUp(int button, int x, int y);
	///	Called when a mouse move event is received.
	void mouseMove(int x, int y);

	///	Returns the name of this state.
	const std::string getName() const {return "GamePlay";};
  private:
	///	Draws the team selection screen.
	void drawTeamSelection(Drawer *d);
	///	Draws the overworld.
	void drawOverworld(Drawer *d);
	///	Draws the battle screen.
	void drawBattle(Drawer *d);
	///	Draws the inventory.
	void drawInventory(Drawer *d);

	///	Called to increment turn counter.
	void incrementTurnCounter();
	///	Called at the end of a 'turn' on the battle screen.
	/*!
		Moves humans, potentially attacks Animals, checks if the battle is over.
	 */
	void battleTurn();
	///	Called to move an Animal or have it attack a Human.
	void battleAnimalMove(std::shared_ptr<AnimalActor>& animal, const TwoFloats& pos);

	///	Helper method to create the maps, add HumanActors etc.
	void createMaps();
	///	Helper method to create AnimalActors.
	std::shared_ptr<AnimalActorOver> createAnimal(int index);
	///	Helper method, returns a tile position if 2 Actors are on the same one.
	/*!
		\return Returns (-1, -1) if no Actors are sharing tiles.
	 */
	TwoFloats getConflictTile();
	///	Helper method to calculate whether an Animal is standing on a Lake tile.
	bool isOnLake(const TwoFloats& pos);

	///	The background texture.
	static uint32_t backTexture;
	///	The mouse cursor.
	static uint32_t mouseImage;
	///	The title image.
	static uint32_t titleImage;
	///	The animal images.
	static uint32_t animalImages[6];
	///	The human images.
	static uint32_t humanImages[3];
	///	The Item images.
	static uint32_t foodImage;
	///	The Item images.
	static uint32_t statBoostImage;
	///	The backpack image.
	static uint32_t backpackImage;
	///	The tile selector image.
	static uint32_t selectorImage;
	///	The button image.
	static uint32_t buttonImage;
	///	The healthbar image.
	static uint32_t healthMeterTopImage;
	///	The healthbar image.
	static uint32_t healthMeterBottomImage;
	///	The healthbar image.
	static uint32_t healthMeterBarImage;
	///	The heart image.
	static uint32_t heartImage;
	///	The equipslot image.
	static uint32_t equipSlotImage;
	///	The message box image.
	static uint32_t messageBoxImage;
	///	The game over message.
	static uint32_t gameOverImage;
	///	The battle message.
	static uint32_t battleImage;
	///	The win1 message.
	static uint32_t win1Image;
	///	The win2 message.
	static uint32_t win2Image;
	///	The font we use.
	static uint32_t font;

	///	To identify the animals.
	enum
	{
		Elephant = 0,
		Lion,
		Hippo,
		Giraffe,
		Owl,
		Wallaby
	};

	///	To identify the humans.
	enum
	{
		ParkWarden = 0,
		Police,
		Soldier
	};

	///	How many turns it takes to escape the Humans.
	enum
	{
		NumWinningTurns = 50
	};

	///	True if the player has yet to select their team.
	bool selectingTeam;
	///	Which animals the player has already selected.
	bool selectedAnimals[6];
	///	How many animals the player has selected.
	int numSelected;

	///	True if the player's currently exploring the overworld.
	bool overworld;
	///	True if the player's currently examining the inventory of one of their animals.
	bool inventory;

	///	The current mouse position.
	TwoFloats mousePos;

	///	Used to display Actor stats in a tooltip.
	StatsDisplay statsDisplay;
	///	The message box used to display messages to the player.
	MessageBox messageBox;
	///	Any DamageTexts on screen.
	std::vector<std::shared_ptr<DamageText> > damageTexts;

	///	The overworld map.
	OverworldMap::Map overworldMap;
	///	The battle maps for all the overworld tiles.
	OverworldBattleMaps battleMaps;
	///	The animals at the player's disposal.
	std::vector<std::shared_ptr<AnimalActorOver> > animals;
	///	The humans in play.
	std::vector<std::shared_ptr<HumanActorOver> > humans;

	///	Which map we're currently battling in.
	TwoFloats currentBattle;
	///	Which map we're currently battling in.
	int battleIndex;
	///	The Overworld AnimalActor involved in the current battle.
	std::shared_ptr<AnimalActorOver> battleAnimalOver;
	///	The Overworld AnimalActor involved in the current battle.
	std::shared_ptr<HumanActorOver> battleHumanOver;
	///	The AnimalActors involved in the current battle.
	std::vector<std::shared_ptr<AnimalActor> > battleAnimals;
	///	The HumanActors involved in the current battle.
	std::vector<std::shared_ptr<HumanActor> > battleHumans;

	///	Items hidden on the overworld map.
	std::vector<std::shared_ptr<Item> > overworldItems;
	///	Items hidden on the battle maps.
	std::vector<std::shared_ptr<Item> > battleItems[OverworldMap::MapWidth*OverworldMap::MapHeight];

	///	The main turn counter.
	int turnCounter;

	///	Used to ensure we add 3 Park Wardens.
	int numWardens;

	///	Which of the animals in the overworld is selected (if any).
	int overworldSelectedAnimal;
	///	Which of the animals in the battle is selected (if any).
	int battleSelectedAnimal;

	///	Game status.
	enum GameStatus
	{
		InPlay = 0,
		GameOver,
		Win1,
		Win2
	};
	///	Game status.
	GameStatus status;

	///	Non-zero if we're display the battle message.
	float battleDisplay;
};

#endif
