//	OpenGLDrawer.h - OpenGL drawing backend (using SDL headers).
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef OPENGLDRAWER_H_
#define OPENGLDRAWER_H_

#include "Drawer.h"
#include "OpenGLFont.h"
#include "OpenGLHeaders.h"

#include <vector>
#include <map>

///	OpenGL drawing backend (using SDL headers).
class OpenGLDrawer : public Drawer
{
  public:
	///	Constructor.
	OpenGLDrawer(int16_t w = 1024, int16_t h = 768);
	///	Destructor.
	~OpenGLDrawer();

	///	Initialise the various OpenGL settings.
	/*!
		\param noTexInterp Whether to interpolate textures when they're scaled.
	 */
	bool initialise(uint16_t width,
					uint16_t height,
					bool fs,
					int aa);
	///	Lightweight initialise method.
	/*!
		\param width The width of the window/screen to draw to.
		\param height The height of the window/screen to draw to.

		This is to be used in situations where the app's window/context has
		already been created.  All it really does is let you set your width &
		height variables.
	 */
	void initialiseLight(uint16_t width, uint16_t height);
	///	Sets whether to interpolate scaled textures (default) or not.
	void setTexInterp(bool val);

	///	Loads an image onto the GPU.
	/*!
		If the image's dimensions are not power-of-two, it will be padded so we
		don't run into problems.  We copy the image's border pixels so the
		texture will tile correctly (if we just blank the extra pixels, it
		creates visible seams when we run at a resolution other than 1024x768).
		The Image structure will then keep track of the actual size of the
		image, and only draw the original rectangle.
	 */
	uint32_t loadImage(uint8_t *data, uint32_t width, uint32_t height);
	///	Loads an image to be used as a glyph.
	void loadGlyph(Glyph g, uint8_t *data, uint32_t size);
	///	Calls glDeleteTextures() on the relevant image.
	void releaseImage(uint32_t id);
	///	Loads a polygon, returns its identifier.
	uint32_t loadPoly(float *points, int numPoints);
	///	Loads a shader, returns its identifier.
	uint32_t loadShader(const std::string& vertex, const std::string& fragment);
	///	Returns whether or not shaders are supported.
	bool getShadersAreSupported() const {return shadersAreSupported;};
	///	Loads a font, returns its identifier.
	uint32_t loadFont(const std::string& name, float size);

	///	Loads the identity matrix etc.
	void beginDrawing();
	///	Swaps buffers.
	void endDrawing();

	///	Returns the contents of the screen as a buffer of pixels.
	unsigned char *getScreenPixels();

	///	Activates a shader to be used for the following operations.
	void useShader(uint32_t shader, float param = 0.0f);
	///	Draws the passed in image on top of the existing one for any further drawing operations.
	void useSecondImage(uint32_t image);
	///	Sets the blend mode to use.
	/*!
		To be called between a beginDrawing()/endDrawing() pair.
	 */
	void setBlendMode(BlendMode val);

	///	Draws the image as a texture-mapped quad(?).
	void drawImage(uint32_t id,
				   float x,
				   float y,
				   float width = 1.0f,
				   float height = 1.0f,
				   float r = 1.0f,
				   float g = 1.0f,
				   float b = 1.0f,
				   float a = 1.0f);
	///	Draws a rotated image.
	void drawImageRot(uint32_t id,
					  float x,
					  float y,
					  float angle,
					  float rotX = 0.0f,
					  float rotY = 0.0f,
					  float width = 1.0f,
					  float height = 1.0f,
					  float r = 1.0f,
					  float g = 1.0f,
					  float b = 1.0f,
					  float a = 1.0f);
	///	Draws part of an image at the desired position.
	void drawSubImage(uint32_t id,
					  float x,
					  float y,
					  float subX,
					  float subY,
					  float subWidth,
					  float subHeight,
					  float width = 1.0f,
					  float height = 1.0f,
					  float r = 1.0f,
					  float g = 1.0f,
					  float b = 1.0f,
					  float a = 1.0f);
	///	Draws a textured polygon at the desired position.
	void drawPoly(uint32_t polyId,
				  uint32_t textureId,
				  float x,
				  float y,
				  float width = 1.0f,
				  float height = 1.0f,
				  float r = 1.0f,
				  float g = 1.0f,
				  float b = 1.0f,
				  float a = 1.0f);
	///	Draws a string as a serious of texture-mapped quads.
	void drawText(const std::string& text,
				  float x,
				  float y,
				  float height,
				  float r = 1.0f,
				  float g = 1.0f,
				  float b = 1.0f,
				  float a = 1.0f);
	///	Draws a string with the selected font.
	void drawText(const std::string& text,
				  uint32_t font,
				  float x,
				  float y,
				  float boxWidth,
				  float scale = 1.0f,
				  float r = 1.0f,
				  float g = 1.0f,
				  float b = 1.0f,
				  float a = 1.0f);
	///	Draws a line between 2 points.
	void drawLine(float x1,
				  float y1,
				  float x2,
				  float y2,
				  float r = 1.0f,
				  float g = 1.0f,
				  float b = 1.0f,
				  float a = 1.0f);
	///	Fills a rectangle with the specfied colour.
	void fillRect(float x,
				  float y,
				  float w,
				  float h,
				  uint8_t r,
				  uint8_t g,
				  uint8_t b,
				  uint8_t a);
	///	Fills a rectangle with the specfied gradient.
	void fillGradient(float x,
					  float y,
					  float w,
					  float h,
					  uint8_t r1,
					  uint8_t g1,
					  uint8_t b1,
					  uint8_t a1,
					  uint8_t r2,
					  uint8_t g2,
					  uint8_t b2,
					  uint8_t a2);

	///	Translates all subsequent drawing commands by the specified offset.
	void translate(float x, float y);

	///	Sets a viewport to restrict drawing to.
	void setViewport(int16_t x, int16_t y, int16_t w, int16_t h);
	///	Resets the viewport.
	void resetViewport();

	///	Returns the width of the available viewport for drawing into.
	int16_t getActualWidth() const {return (int16_t)viewW;};
	///	Returns the height of the available viewport for drawing into.
	int16_t getActualHeight() const {return (int16_t)viewH;};
	///	Returns the x-axis offset of the main viewport we're drawing in.
	int16_t getViewportOffsetX() const {return (int16_t)viewX;};
	///	Returns the y-axis offset of the main viewport we're drawing in.
	int16_t getViewportOffsetY() const {return (int16_t)viewY;};
  private:
	///	Used so that graphics cards which don't support OpenGL shaders don't crash.
	static void GLAPIENTRY nullUniform1i(GLint, GLint);
	///	Used so that graphics cards which don't support OpenGL shaders don't crash.
	static void GLAPIENTRY nullUniform1f(GLint, GLfloat);
	///	Used so that graphics cards which don't support OpenGL shaders don't crash.
	static GLuint GLAPIENTRY nullCreateShaderObject(GLenum);
	///	Used so that graphics cards which don't support OpenGL shaders don't crash.
	static void GLAPIENTRY nullShaderSource(GLuint, GLsizei, const GLchar **, const GLint *);
	///	Used so that graphics cards which don't support OpenGL shaders don't crash.
	static void GLAPIENTRY nullCompileShader(GLuint);
	///	Used so that graphics cards which don't support OpenGL shaders don't crash.
	static GLuint GLAPIENTRY nullCreateProgramObject(void);
	///	Used so that graphics cards which don't support OpenGL shaders don't crash.
	static void GLAPIENTRY nullAttachObject(GLuint, GLuint);
	///	Used so that graphics cards which don't support OpenGL shaders don't crash.
	static void GLAPIENTRY nullLinkProgram(GLuint);
	///	Used so that graphics cards which don't support OpenGL shaders don't crash.
	static GLint GLAPIENTRY nullGetUniformLocation(GLuint, const GLchar *);
	///	Used so that graphics cards which don't support OpenGL shaders don't crash.
	static void GLAPIENTRY nullUseProgramObject(GLuint);

	///	Helper method to check whether we're running on a 16-bit display.
	bool is16Bit();

	///	Helper method to initialise the glyphs to blank textures.
	void initFonts();
	///	Helper method to test whether a number is a power-of-two.
	bool isPowerOfTwo(uint32_t val);
	///	Helper method to return the next highest number that is a power-of-two.
	/*!
		\return if val is already power-of-two, it will be returned as-is.
	 */
	uint32_t powerOfTwo(uint32_t val);

	///	Helper method to setup the shader function pointers.
	void setupShaderFunctions();

	///	Helper method.  Loads a shader file and returns it as a string.
	std::string loadShaderFile(const std::string& path);
	///	Helper method.  Streams the info log to standard out.
	void printInfoLog(unsigned int obj);

	///	Structure representing an OpenGL image.
	struct Image
	{
		///	The texture to use.
		GLuint texture;
		///	The width of the image.
		uint16_t width;
		///	The height of the image.
		uint16_t height;
		///	The width of the texture.
		float texWidth;
		///	The height of the texture.
		float texHeight;
	};

	///	Structure representing a polygon.
	struct Polygon
	{
		///	The polygon's points.
		float *xPoints;
		///	The polygon's points.
		float *yPoints;
		///	The polygon's texture coords.
		float *xTex;
		///	The polygon's texture coords.
		float *yTex;
		///	The number of points in the polygon.
		int numPoints;
	};

	///	The texture used to draw lines.
	static uint32_t lineImage;

	///	The images currently loaded and ready to be drawn.
	std::map<uint32_t, Image> images;
	///	The polygons currently loaded and ready to be drawn.
	std::vector<Polygon> polys;
	///	The glyphs to draw text with.
	Image glyphs[NumGlyphs];
	//If we're using non-power-of-2 textures for the fonts, this makes sure
	//they're spaced correctly.
	int glyphSpace;
	///	The loaded truetype fonts.
	std::vector<OpenGLFont *> fonts;

	///	The second texture if we're doing multitexturing.
	uint32_t secondImage;
	///	The location of the main texture (for shaders).
	/*!
		For this to work, the uniform variable in the fragment shader *must*
		be 'main'.
	 */
	uint32_t tex1Loc;
	///	The location of the second texture (for shaders).
	/*!
		For this to work, the uniform variable in the fragment shader *must*
		be 'second'.
	 */
	uint32_t tex2Loc;
	///	The location of the optional shader parameter.
	/*!
		For this to work, the uniform variable in the fragment shader *must*
		be 'shaderParam'.
	 */
	uint32_t paramLoc;

	///	The current max identifier for images.
	uint32_t imagesMaxId;

	///	The width of our screen.
	int16_t screenWidth;
	///	The height of our screen.
	int16_t screenHeight;
	///	The left side of our viewport.
	float viewX;
	///	The bottom side of our viewport.
	float viewY;
	///	The width of our viewport.
	float viewW;
	///	The height of our viewport.
	float viewH;

	///	Whether to interpolate textures when they're scaled.
	bool texInterp;

	///	If shaders are supported or not.
	bool shadersAreSupported;
	///	Shader functionality function pointers.
	void (GLAPIENTRY *shaderUniform1i)(GLint, GLint);
	///	Shader functionality function pointers.
	void (GLAPIENTRY *shaderUniform1f)(GLint, GLfloat);
	///	Shader functionality function pointers.
	GLuint (GLAPIENTRY *shaderCreateShaderObject)(GLenum);
	///	Shader functionality function pointers.
	void (GLAPIENTRY *shaderShaderSource)(GLuint, GLsizei, const GLchar **, const GLint *);
	///	Shader functionality function pointers.
	void (GLAPIENTRY *shaderCompileShader)(GLuint);
	///	Shader functionality function pointers.
	GLuint (GLAPIENTRY *shaderCreateProgramObject)(void);
	///	Shader functionality function pointers.
	void (GLAPIENTRY *shaderAttachObject)(GLuint, GLuint);
	///	Shader functionality function pointers.
	void (GLAPIENTRY *shaderLinkProgram)(GLuint);
	///	Shader functionality function pointers.
	GLint (GLAPIENTRY *shaderGetUniformLocation)(GLuint, const GLchar *);
	///	Shader functionality function pointers.
	void (GLAPIENTRY *shaderUseProgramObject)(GLuint);

	///	Image data used to draw lines.
	static const char* line_png;
	///	Image data used to draw lines.
    const int line_pngSize;// = 144;
};

#endif
