//	OpenGLVBODrawer.cpp - OpenGL drawing backend using VBOs.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2010  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "OpenGLVBODrawer.h"
#include "ImageLoader.h"
#include "DebugStatement.h"

#include "SDL.h"
#include <cmath>
#include <cassert>
#include <fstream>
#include <iostream>

#ifdef DEBUG
#include <sstream>
#endif

using namespace std;

#define BUFFER_OFFSET(i) ((char *)NULL + (i))

//------------------------------------------------------------------------------
uint32_t OpenGLVBODrawer::lineImage = 0;

//------------------------------------------------------------------------------
OpenGLVBODrawer::OpenGLVBODrawer(int16_t w, int16_t h):
Drawer(w, h),
glyphSpace(0),
secondImage(0),
imagesMaxId(1),
screenWidth(1024),
screenHeight(768),
viewX(0.0f),
viewY(0.0f),
viewW(1024.0f),
viewH(768.0f),
texInterp(true),
shadersAreSupported(true),
line_pngSize(144)
{
	//Initialise SDL_TTF.
	TTFSingleton::getInstance();
}

//------------------------------------------------------------------------------
OpenGLVBODrawer::~OpenGLVBODrawer()
{
	int i;

	for(i=0;i<polys.size();++i)
	{
		delete [] polys[i].xPoints;
		delete [] polys[i].yPoints;
		delete [] polys[i].xTex;
		delete [] polys[i].yTex;
	}
}

//------------------------------------------------------------------------------
bool OpenGLVBODrawer::initialise(uint16_t width,
							  uint16_t height,
							  bool fs,
							  int aa)
{
	GLenum err;
	bool retval = true;
	int flags = SDL_OPENGL;
	const float dW = getDrawWidth();
	const float dH = getDrawHeight();
	const float dAspectRatio = dH/dW;
	const float aspectRatio = (float)height/(float)width;

	screenWidth = width;
	screenHeight = height;

	//Set OpenGL attributes (we use 32 bit colour if we can).
	if(is16Bit())
	{
		cout << "OpenGLVBODrawer: Display is 16bit." << endl;
		if(SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5) < 0)
			cout << "OpenGLVBODrawer: Could not set red pixel size to 5..." << endl;
		if(SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 5) < 0)
			cout << "OpenGLVBODrawer: Could not set green pixel size to 5..." << endl;
		if(SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5) < 0)
			cout << "OpenGLVBODrawer: Could not set blue pixel size to 5..." << endl;
	}
	else
	{
		cout << "OpenGLVBODrawer: Display is 32bit." << endl;
		if(SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 8) < 0)
			cout << "OpenGLVBODrawer: Could not set red pixel size to 8..." << endl;
		if(SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 8) < 0)
			cout << "OpenGLVBODrawer: Could not set green pixel size to 8..." << endl;
		if(SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 8) < 0)
			cout << "OpenGLVBODrawer: Could not set blue pixel size to 8..." << endl;
		if(SDL_GL_SetAttribute(SDL_GL_ALPHA_SIZE, 8) < 0)
			cout << "OpenGLVBODrawer: Could not set alpha pixel size to 8..." << endl;
	}
	if(SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16) < 0)
		cout << "OpenGLVBODrawer: Could not set depth size to 16..." << endl;
	if(SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1) < 0)
		cout << "OpenGLVBODrawer: Could not set double buffering..." << endl;
	if(SDL_GL_SetAttribute(SDL_GL_SWAP_CONTROL, 1) < 0)
		cout << "OpenGLVBODrawer: Could not set vertical sync..." << endl;
	if(aa > 0)
	{
		if(SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 1) < 0)
			cout << "OpenGLVBODrawer: Could not set multi sample buffers..." << endl;
		if(SDL_GL_SetAttribute(SDL_GL_MULTISAMPLESAMPLES, aa) < 0)
			cout << "OpenGLVBODrawer: Could not set multi sample samples..." << endl;
	}

	if(fs)
		flags |= SDL_FULLSCREEN;
	else
	{
		char tempstr[] = "SDL_VIDEO_CENTERED=1";
		putenv(tempstr); 
	}

	//Construct window etc.
	if(SDL_SetVideoMode(width, height, 0, flags) == 0)
	{
#ifdef DEBUG
		cout << "SDLDrawer: Could not set video mode: ";
		cout << SDL_GetError() << endl;
#endif
		retval = false;
	}
	else
		retval = true;

	//Initialise GLEW.
	err = glewInit();
	if(err != GLEW_OK)
		cout << "GLEW error: " << glewGetErrorString(err) << endl;
	else
		cout << "Using GLEW version: " << glewGetString(GLEW_VERSION) << endl;

	setupShaderFunctions();

#ifdef DEBUG_STATEMENTS
	{
		GLint tempint;
		glGetIntegerv(GL_MAX_TEXTURE_SIZE, &tempint);
		DebugStatement("GL_MAX_TEXTURE_SIZE = " << tempint);

		glTexImage2D(GL_PROXY_TEXTURE_2D,
						 0,
						 GL_RGBA,
						 2048,
						 2048,
						 0,
						 GL_RGBA,
						 GL_UNSIGNED_BYTE,
						 NULL);
		glGetTexLevelParameteriv(GL_PROXY_TEXTURE_2D, 0, GL_TEXTURE_WIDTH, &tempint);
		DebugStatement("GL_TEXTURE_WIDTH = " << tempint);
	}
#endif

	//Initialise the fonts.
	initFonts();

	//First setup the full window as a viewport.
	glViewport(0, 0, width, height);

	//Clear to black (if we're widescreen we'll have black bars at the sides).
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT);

	//If we're widescreen, set the viewport to be a centred 4:3 aspect ratio.
	if(aspectRatio != dAspectRatio)
	{
		if(aspectRatio < dAspectRatio)
		{
			//viewW = (4.0f/3.0f) * height;
			viewW = (dW/dH) * height;
			viewH = height;
			viewX = (width-viewW)*0.5f;
			viewY = 0;
		}
		else if(aspectRatio > dAspectRatio) //Seems unlikely, but we might as well
		{							 //cover it...
			viewW = width;
			viewH = width * dAspectRatio;
			viewX = 0.0f;
			viewY = (height-viewH)*0.5f;
		}

#ifdef DEBUG
		cout << "OpenGLVBODrawer: Setting viewport to: " << x << "x";
		cout << y << " : " << w << "x" << h << endl;
#endif

		glViewport((int)viewX, (int)viewY, (int)viewW, (int)viewH);
	}
	else
	{
		viewW = width;
		viewH = height;
	}

	//Now set up everything else.
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	//So our co-ordinates for drawing are 0-1024 (x) and 768-0 (y).
	gluOrtho2D(0.0f, dW, dH, 0.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glShadeModel(GL_SMOOTH);
	glClearDepth(1.0f);
	glDisable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glEnable(GL_TEXTURE_2D);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);

	//Load the line image.
	if(!lineImage)
	{
		ImageLoader imLoad;
		ImageData data;

		data = imLoad.loadImage(line_png, line_pngSize);
		lineImage = loadImage(data.data, data.width, data.height);
	}

	//Create imageVBO.
	{
		GLfloat vertices[] = {0.0f, 0.0f,
							  1.0f, 0.0f,
							  1.0f, 1.0f,
							  1.0f, 1.0f,
							 0.0f, 1.0f,
							  0.0f, 0.0f};
		GLfloat texCoords[] = {0.0f, 0.0f,
							   1.0f, 0.0f,
							   1.0f, 1.0f,
							   1.0f, 1.0f,
							   0.0f, 1.0f,
							   0.0f, 0.0f};

		glGenBuffers(1, &imageVBO);
		glBindBuffer(GL_ARRAY_BUFFER, imageVBO);
		glBufferData(GL_ARRAY_BUFFER, sizeof(GLfloat)*24, 0, GL_STREAM_DRAW);

		glBufferSubData(GL_ARRAY_BUFFER, 0, sizeof(GLfloat)*12, texCoords);
		glBufferSubData(GL_ARRAY_BUFFER,
						sizeof(GLfloat)*12,
						sizeof(GLfloat)*12,
						vertices);

		glTexCoordPointer(2, GL_FLOAT, 0, 0);
		glVertexPointer(2, GL_FLOAT, 0, BUFFER_OFFSET(sizeof(GLfloat)*12));

		glBindBuffer(GL_ARRAY_BUFFER, 0);
	}

	return retval;
}

//------------------------------------------------------------------------------
void OpenGLVBODrawer::initialiseLight(uint16_t width, uint16_t height)
{
	const float dW = getDrawWidth();
	const float dH = getDrawHeight();
	const float dAspectRatio = dH/dW;
	const float aspectRatio = (float)height/(float)width;

	screenWidth = width;
	screenHeight = height;

	//Initialise the fonts.
	initFonts();

	//If we're widescreen, set the viewport to be a centred 4:3 aspect ratio.
	if(aspectRatio != dAspectRatio)
	{
		if(aspectRatio < dAspectRatio)
		{
			//viewW = (4.0f/3.0f) * height;
			viewW = (dW/dH) * height;
			viewH = height;
			viewX = (width-viewW)*0.5f;
			viewY = 0;
		}
		else if(aspectRatio > dAspectRatio) //Seems unlikely, but we might as well
		{							 //cover it...
			viewW = width;
			viewH = width * dAspectRatio;
			viewX = 0.0f;
			viewY = (height-viewH)*0.5f;
		}

#ifdef DEBUG
		cout << "OpenGLVBODrawer: Setting viewport to: " << x << "x";
		cout << y << " : " << w << "x" << h << endl;
#endif

		glViewport((int)viewX, (int)viewY, (int)viewW, (int)viewH);
	}
	else
	{
		viewW = width;
		viewH = height;
	}

	//Now set up everything else.
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	//So our co-ordinates for drawing are 0-1024 (x) and 768-0 (y).
	gluOrtho2D(0.0f, dW, dH, 0.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glShadeModel(GL_SMOOTH);
	glClearDepth(1.0f);
	glDisable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glEnable(GL_TEXTURE_2D);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);

	//Load the line image.
	if(!lineImage)
	{
		ImageLoader imLoad;
		ImageData data;

		data = imLoad.loadImage(line_png, line_pngSize);
		lineImage = loadImage(data.data, data.width, data.height);
	}

	//Create imageVBO.
	{
		GLfloat vertices[] = {0.0f, 0.0f,
							  1.0f, 0.0f,
							  1.0f, 1.0f,
							  1.0f, 1.0f,
							  0.0f, 1.0f,
							  0.0f, 0.0f};
		GLfloat texCoords[] = {0.0f, 0.0f,
							   1.0f, 0.0f,
							   1.0f, 1.0f,
							   1.0f, 1.0f,
							   0.0f, 1.0f,
							   0.0f, 0.0f};

		glGenBuffers(1, &imageVBO);
		glBindBuffer(GL_ARRAY_BUFFER, imageVBO);
		glBufferData(GL_ARRAY_BUFFER, sizeof(GLfloat)*12, 0, GL_STREAM_DRAW);

		glBufferSubData(GL_ARRAY_BUFFER, 0, sizeof(GLfloat)*6, texCoords);
		glBufferSubData(GL_ARRAY_BUFFER,
						sizeof(GLfloat)*6,
						sizeof(GLfloat)*6,
						vertices);

		glTexCoordPointer(2, GL_FLOAT, 0, 0);
		glVertexPointer(2, GL_FLOAT, 0, BUFFER_OFFSET(sizeof(GLfloat)*6));

		glBindBuffer(GL_ARRAY_BUFFER, 0);
	}
}

//------------------------------------------------------------------------------
void OpenGLVBODrawer::setTexInterp(bool val)
{
	texInterp = val;
}

//------------------------------------------------------------------------------
uint32_t OpenGLVBODrawer::loadImage(uint8_t *data, uint32_t width, uint32_t height)
{
	Image tempIm;
	uint32_t retval = 0;

	if(data)
	{
		//If it's not a power-of-two-sized image, we need to make it so.
		if((!isPowerOfTwo(width)) || (!isPowerOfTwo(height)))
		{
			uint32_t i;
			uint32_t x, y;
			uint8_t tempint;
			const uint32_t w = powerOfTwo(width);
			const uint32_t h = powerOfTwo(height);
			const uint32_t dataSize = w*h*4;
			uint8_t *tempData = new uint8_t[dataSize];

			//Blank the new image.
			for(i=0;i<dataSize;++i)
				tempData[i] = 0;

			//Copy the old image into the upper left corner.
			i = 0;
			for(y=0;y<height;++y)
			{
				for(x=0;x<(width*4);++x)
				{
					tempData[x+(y*w*4)] = data[i];
					++i;
				}
				//Copy the previous value along to fill out the texture.
				tempint = data[i-1];
				for(x=(width*4);x<(w*4);++x)
					tempData[x+(y*w*4)] = tempint;
			}

			//Copy the previous value along to fill out the texture.
			i = (width*height*4) - (width*4); //So we're back to reading the previous line.
			for(x=0;x<(w*4);++x)
			{
				tempint	= data[i];
				for(y=height;y<h;++y)
					tempData[x+(y*w*4)] = tempint;
				if(i < ((width*4)-1))
					++i;
			}

			//Inform tempIm of the dimensions.
			tempIm.width = width;
			tempIm.height = height;
			tempIm.texWidth = (float)width/(float)w;
			tempIm.texHeight = (float)height/(float)h;

#ifdef DEBUG
			cout << "OpenGLVBODrawer: Image not power-of-two: " << (unsigned int)width << "x" << (unsigned int)height << endl;
			cout << "\tTexture: " << (unsigned int)w << "x" << (unsigned int)h << endl;
#endif

			//Delete the old data.
			delete [] data;
			//Make sure data now points to the new data.
			data = tempData;
		}
		else
		{
			tempIm.width = width;
			tempIm.height = height;
			tempIm.texWidth = 1.0f;
			tempIm.texHeight = 1.0f;
		}

		//Create the texture.
		glGenTextures(1, &(tempIm.texture));

		glBindTexture(GL_TEXTURE_2D, tempIm.texture);

		glTexImage2D(GL_TEXTURE_2D,
					 0,
					 GL_RGBA,
					 powerOfTwo(tempIm.width),
					 powerOfTwo(tempIm.height),
					 0,
					 GL_RGBA,
					 GL_UNSIGNED_BYTE,
					 data);

		if(texInterp)
		{
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		}
		else
		{
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		}
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

		glBindTexture(GL_TEXTURE_2D, 0);

		//Get rid of the image data.
		delete [] data;

		//Add tempIm to our list of images.
		while(images.find(imagesMaxId) != images.end())
			++imagesMaxId;
		retval = imagesMaxId;

		images.insert(make_pair(retval, tempIm));
	}
#ifdef DEBUG
	else
		cout << "OpenGLVBODrawer::loadImage(): No data passed in!" << endl;
#endif

	return retval;
}

//------------------------------------------------------------------------------
void OpenGLVBODrawer::loadGlyph(Glyph g, uint8_t *data, uint32_t size)
{
	Image tempIm;

	//If it's not a power-of-two-sized image, we need to make it so.
	if(!isPowerOfTwo(size))
	{
		uint32_t i;
		uint32_t x, y;
		uint8_t tempint;
		const uint32_t s = powerOfTwo(size);
		const uint32_t dataSize = s*s*4;
		uint8_t *tempData = new uint8_t[dataSize];

		//Setup glyphSpace.
		glyphSpace = s-size;

		//Blank the new image.
		for(i=0;i<dataSize;++i)
			tempData[i] = 0;

		//Copy the old image into the upper left corner.
		i = 0;
		for(y=0;y<size;++y)
		{
			for(x=0;x<(size*4);++x)
			{
				tempData[x+(y*s*4)] = data[i];
				++i;
			}
			//Copy the previous value along to fill out the texture.
			tempint = data[i-1];
			for(x=(size*4);x<(s*4);++x)
				tempData[x+(y*s*4)] = tempint;
		}
		//Copy a transparent pixel to fill out the texture.
		i -= (size*4); //So we're back to reading the previous line.
		for(x=0;x<(s*4);++x)
		{
			tempint	= data[i];
			for(y=size;y<s;++y)
				tempData[x+(y*s*4)] = 0;
			++i;
		}

#ifdef DEBUG
		cout << "OpenGLVBODrawer: Glyph not power-of-two: " << (unsigned int)size << "x" << (unsigned int)size << endl;
		cout << "\tTexture: " << (unsigned int)s << "x" << (unsigned int)s << endl;
#endif

		size = s;

		//Delete the old data.
		delete [] data;
		//Make sure data now points to the new data.
		data = tempData;
	}

	if(data)
	{
		tempIm.width = size;
		tempIm.height = size;
		tempIm.texWidth = 1.0f;
		tempIm.texHeight = 1.0f;

		//Create the texture.
		glGenTextures(1, &(tempIm.texture));

		glBindTexture(GL_TEXTURE_2D, tempIm.texture);

		glTexImage2D(GL_TEXTURE_2D,
					 0,
					 GL_RGBA,
					 tempIm.width,
					 tempIm.height,
					 0,
					 GL_RGBA,
					 GL_UNSIGNED_BYTE,
					 data);

		if(texInterp)
		{
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		}
		else
		{
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		}

		//Get rid of the existing glyph texture.
		//cout << "glIsTexture(): " << (bool)glIsTexture(glyphs[g].texture) << endl;
		glDeleteTextures(1, &(glyphs[g].texture));

		//Copy across the new one.
		glyphs[g].texture = tempIm.texture;
		glyphs[g].width = tempIm.width;
		glyphs[g].height = tempIm.height;
		glyphs[g].texWidth = tempIm.texWidth;
		glyphs[g].texHeight = tempIm.texHeight;
	}
#ifdef DEBUG
	else
		cout << "OpenGLVBODrawer::loadGlyph(): No data passed in!" << endl;
#endif
}

//------------------------------------------------------------------------------
void OpenGLVBODrawer::releaseImage(uint32_t id)
{
	map<uint32_t, Image>::iterator it = images.find(id);

	if(it != images.end())
	{
		glDeleteTextures(1, &(it->second.texture));
		images.erase(it);
	}
#ifdef DEBUG
	else
		cout << "OpenGLVBODrawer: Could not release image: " << (unsigned int)id << endl;
#endif
}

//------------------------------------------------------------------------------
uint32_t OpenGLVBODrawer::loadPoly(float *points, int numPoints)
{
	int i;
	Polygon p;
	float avgX = 0.0f;
	float avgY = 0.0f;
	float left = 10000.0f;
	float right = -10000.0f;
	float top = 10000.0f;
	float bottom = -10000.0f;

	p.xPoints = new float[numPoints+1];
	p.yPoints = new float[numPoints+1];
	p.xTex = new float[numPoints+1];
	p.yTex = new float[numPoints+1];
	p.numPoints = numPoints+1;

	for(i=0;i<numPoints;++i)
	{
		avgX += points[(i*2)];
		avgY += points[(i*2)+1];

		p.xPoints[i+1] = points[(i*2)];
		p.yPoints[i+1] = points[(i*2)+1];

		if(p.xPoints[i+1] < left)
			left = p.xPoints[i+1];
		else if(p.xPoints[i+1] > right)
			right = p.xPoints[i+1];

		if(p.yPoints[i+1] < top)
			top = p.yPoints[i+1];
		else if(p.yPoints[i+1] > bottom)
			bottom = p.yPoints[i+1];
	}
	avgX /= (float)numPoints;
	avgY /= (float)numPoints;

	p.xPoints[0] = avgX;
	p.yPoints[0] = avgY;

	for(i=0;i<p.numPoints;++i)
	{
		p.xTex[i] = (p.xPoints[i] - left)/(right-left);
		p.yTex[i] = (p.yPoints[i] - top)/(bottom-top);
	}

	polys.push_back(p);

	return polys.size()-1;
}

//------------------------------------------------------------------------------
uint32_t OpenGLVBODrawer::loadShader(const string& vertex, const string& fragment)
{
	uint32_t vertexObject;
	uint32_t fragmentObject;
	uint32_t retval = 0;

	if(!shadersAreSupported)
		return 0;

	//Create the shader objects.
	vertexObject = shaderCreateShaderObject(GL_VERTEX_SHADER_ARB);
	fragmentObject = shaderCreateShaderObject(GL_FRAGMENT_SHADER_ARB);

	//Read the shaders from their files.
	string vertexStr = loadShaderFile(vertex);
	string fragmentStr = loadShaderFile(fragment);
	const char *vertexText = vertexStr.c_str();
	const char *fragmentText = fragmentStr.c_str();

	//Read the shader source into the shader objects.
	shaderShaderSource(vertexObject, 1, &vertexText, NULL);
	shaderShaderSource(fragmentObject, 1, &fragmentText, NULL);

	//Compile the shaders.
	shaderCompileShader(vertexObject);
	shaderCompileShader(fragmentObject);

	//Create the program object.
	retval = shaderCreateProgramObject();

	//Attach the shaders to the program.
	shaderAttachObject(retval, vertexObject);
	shaderAttachObject(retval, fragmentObject);

	//Finally, link the program.
	shaderLinkProgram(retval);

	//Output the info log.
	cout << "OpenGLVBODrawer Shader InfoLog:" << endl;
	cout << "----------------------------" << endl;
	cout << vertex << ":" << endl;
	printInfoLog(vertexObject);
	cout << fragment << ":" << endl;
	printInfoLog(fragmentObject);

	//Get the texture variable locations.
	tex1Loc = shaderGetUniformLocation(retval, "base");
	tex2Loc = shaderGetUniformLocation(retval, "second");
	paramLoc = shaderGetUniformLocation(retval, "shaderParam");

	return retval;
}

//------------------------------------------------------------------------------
uint32_t OpenGLVBODrawer::loadFont(const string& name, float size)
{
	OpenGLFont *f = new OpenGLFont(name, size);

	if(f->getValid())
	{
		fonts.push_back(f);
		return fonts.size();
	}
	else
		return 0;
}

//------------------------------------------------------------------------------
void OpenGLVBODrawer::beginDrawing()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
}

//------------------------------------------------------------------------------
void OpenGLVBODrawer::endDrawing()
{
	SDL_GL_SwapBuffers();
	secondImage = 0;
}

//------------------------------------------------------------------------------
unsigned char *OpenGLVBODrawer::getScreenPixels()
{
	unsigned char *retval = new unsigned char[screenWidth*screenHeight*3];

	//So we don't have to call this after Drawer::endDrawing().
	SDL_GL_SwapBuffers();

	//memset(retval, (screenWidth*screenHeight*3), 0);
	memset(retval, 0, (screenWidth*screenHeight*3));
	//Get pixels from opengl frame buffer.
	glReadPixels(0,					//X start pos.
				 0,					//Y start pos.
				 screenWidth,		//Width.
				 screenHeight,		//Height.
				 GL_RGB,			//What data we're reading.
				 GL_UNSIGNED_BYTE,	//Format to write to.
				 (void *)retval);	//Buffer to write to.

	return retval;
}

//------------------------------------------------------------------------------
void OpenGLVBODrawer::useShader(uint32_t shader, float param)
{
	shaderUseProgramObject(shader);
	shaderUniform1f(paramLoc, param);
}

//------------------------------------------------------------------------------
void OpenGLVBODrawer::useSecondImage(uint32_t image)
{
	secondImage = image;
}

//------------------------------------------------------------------------------
void OpenGLVBODrawer::setBlendMode(BlendMode val)
{
	switch(val)
	{
		case AlphaBlend:
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			glBlendEquation(GL_FUNC_ADD);
			break;
		case Additive:
			glBlendFunc(GL_SRC_ALPHA, GL_ONE);
			glBlendEquation(GL_FUNC_ADD);
			break;
		case Subtractive:
			//glBlendFuncSeparate(GL_SRC_COLOR, GL_ONE, GL_ONE, GL_ONE);
			//glBlendEquationSeparate(GL_FUNC_REVERSE_SUBTRACT, GL_FUNC_ADD);
			glBlendFunc(GL_DST_COLOR, GL_SRC_ALPHA);
			glBlendEquation(GL_FUNC_SUBTRACT);
			break;
	}
}

//------------------------------------------------------------------------------
void OpenGLVBODrawer::drawImage(uint32_t id,
							 float x,
							 float y,
							 float width,
							 float height,
							 float r,
							 float g,
							 float b,
							 float a)
{
	Image *image;
	float w, h;
	float texCoords[] = {0.0f, 0.0f,
						 1.0f, 0.0f,
						 1.0f, 1.0f,
						 1.0f, 1.0f,
						 0.0f, 1.0f,
						 0.0f, 0.0f};
	map<uint32_t, Image>::iterator it, it2;

	it = images.find(id);

	assert(it != images.end());

	image = &(it->second);
	w = ((float)image->width * width);
	h = ((float)image->height * height);
	if((image->texWidth < 1.0f) || (image->texHeight < 1.0f))
	{
		texCoords[2] = texCoords[4] = texCoords[6] = image->texWidth;
		texCoords[5] = texCoords[7] = texCoords[9] = image->texHeight;
	}

	glActiveTextureARB(GL_TEXTURE0_ARB);
	glBindTexture(GL_TEXTURE_2D, image->texture);
	shaderUniform1i(tex1Loc, 0);

	if(secondImage)
	{
		Image im;

		it2 = images.find(secondImage);
		assert(it != images.end());

		im = it2->second;

		glActiveTextureARB(GL_TEXTURE1_ARB);
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, im.texture);
		shaderUniform1i(tex2Loc, 1);
	}

	glPushMatrix();
	glTranslatef(x, y, 0.0f);
	glScalef(w, h, 1.0f);

	glColor4f(r, g, b, a);

	glBindBuffer(GL_ARRAY_BUFFER, imageVBO);
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);

	//glBufferSubData(GL_ARRAY_BUFFER, 0, sizeof(GLfloat)*12, texCoords);
	glDrawArrays(GL_TRIANGLES, 0, 12);

	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	glDisableClientState(GL_VERTEX_ARRAY);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	glPopMatrix();

	glActiveTextureARB(GL_TEXTURE1_ARB);
	glBindTexture(GL_TEXTURE_2D, 0);
	glDisable(GL_TEXTURE_2D);
	glActiveTextureARB(GL_TEXTURE0_ARB);
	glBindTexture(GL_TEXTURE_2D, 0);
}

//------------------------------------------------------------------------------
void OpenGLVBODrawer::drawImageRot(uint32_t id,
							    float x,
							    float y,
							    float angle,
							    float rotX,
							    float rotY,
								float width,
								float height,
								float r,
								float g,
								float b,
								float a)
{
	Image *image;
	map<uint32_t, Image>::iterator it;

	it = images.find(id);

	assert(it != images.end());

	image = &(it->second);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();

	//Translate to the position.
	glTranslatef(x, y, 0.0f);

	//Scale the image accordingly.
	glScalef(width, height, 1.0f);

	//Rotate around the rot point.
	glTranslatef(rotX, rotY, 0.0f);
	glRotatef(angle, 0.0f, 0.0f, 1.0f);
	glTranslatef(-rotX, -rotY, 0.0f);

	glBindTexture(GL_TEXTURE_2D, image->texture);

	glColor4f(r, g, b, a);
	glBegin(GL_QUADS);
	{
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 0.0f);
		glVertex2f(0.0f, 0.0f);

		glMultiTexCoord2fARB(GL_TEXTURE0_ARB, image->texWidth, 0.0f);
		glVertex2f((float)image->width, 0.0f);

		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,
							 image->texWidth,
							 image->texHeight);
		glVertex2f((float)image->width, (float)image->height);

		glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, image->texHeight);
		glVertex2f(0.0f, (float)image->height);
	}
	glEnd();
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

	glBindTexture(GL_TEXTURE_2D, 0);

	glPopMatrix();
}

//------------------------------------------------------------------------------
void OpenGLVBODrawer::drawSubImage(uint32_t id,
							    float x,
							    float y,
							    float subX,
							    float subY,
							    float subWidth,
							    float subHeight,
								float width,
								float height,
							    float r,
							    float g,
							    float b,
							    float a)
{
	float w, h;
	Image *image;
	map<uint32_t, Image>::iterator it, it2;

	it = images.find(id);

	assert(it != images.end());

	image = &(it->second);

	glActiveTextureARB(GL_TEXTURE0_ARB);
	glBindTexture(GL_TEXTURE_2D, image->texture);
	shaderUniform1i(tex1Loc, 0);

	if(secondImage)
	{
		Image im;

		it2 = images.find(secondImage);
		if(it2 != images.end())
		{
			im = it2->second;

			glActiveTextureARB(GL_TEXTURE1_ARB);
			glEnable(GL_TEXTURE_2D);
			glBindTexture(GL_TEXTURE_2D, im.texture);
			shaderUniform1i(tex2Loc, 1);
		}
	}

	w = subWidth * image->width * width;
	h = subHeight * image->height * height;
	subX *= image->texWidth;
	subY *= image->texHeight;
	subWidth *= image->texWidth;
	subHeight *= image->texHeight;

	glColor4f(r, g, b, a);
	glBegin(GL_QUADS);
	{
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB, subX, subY);
		glMultiTexCoord2fARB(GL_TEXTURE1_ARB, subX, subY);
		glVertex2f(x, y);

		glMultiTexCoord2fARB(GL_TEXTURE0_ARB, subX+subWidth, subY);
		glMultiTexCoord2fARB(GL_TEXTURE1_ARB, subX+subWidth, subY);
		glVertex2f(x+w, y);

		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,
							 subX+subWidth,
							 subY+subHeight);
		glMultiTexCoord2fARB(GL_TEXTURE1_ARB,
							 subX+subWidth,
							 subY+subHeight);
		glVertex2f(x+w, y+h);

		glMultiTexCoord2fARB(GL_TEXTURE0_ARB, subX, subY+subHeight);
		glMultiTexCoord2fARB(GL_TEXTURE1_ARB, subX, subY+subHeight);
		glVertex2f(x, y+h);
	}
	glEnd();

	glScalef(1.0f, 1.0f, 1.0f);

	glActiveTextureARB(GL_TEXTURE1_ARB);
	glBindTexture(GL_TEXTURE_2D, 0);
	glDisable(GL_TEXTURE_2D);
	glActiveTextureARB(GL_TEXTURE0_ARB);
	glBindTexture(GL_TEXTURE_2D, 0);
}

//------------------------------------------------------------------------------
void OpenGLVBODrawer::drawPoly(uint32_t polyId,
							uint32_t textureId,
							float x,
							float y,
							float width,
							float height,
							float r,
							float g,
							float b,
							float a)
{
	int i;
	Polygon p;
	Image *image;
	float tempX, tempY;
	map<uint32_t, Image>::iterator it, it2;

	it = images.find(textureId);

	if((it != images.end()) || (polyId >= polys.size()))
	{
		p = polys[polyId];
		image = &(it->second);

		tempX = -(((float)image->width * width) - (float)image->width);
		tempX *= 0.5f;
		tempX += x;

		tempY = -(((float)image->height * height) - (float)image->height);
		tempY *= 0.5f;
		tempY += y;

		glActiveTextureARB(GL_TEXTURE0_ARB);
		glEnable(GL_TEXTURE_2D);
		/*glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_EXT);
		glTexEnvf (GL_TEXTURE_ENV, GL_COMBINE_RGB_EXT, GL_REPLACE);*/
		glBindTexture(GL_TEXTURE_2D, image->texture);
		shaderUniform1i(tex1Loc, 0);

		if(secondImage)
		{
			Image im;

			it2 = images.find(secondImage);
			if(it2 != images.end())
			{
				im = it2->second;

				glActiveTextureARB(GL_TEXTURE1_ARB);
				glEnable(GL_TEXTURE_2D);
				/*glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_EXT);
				glTexEnvf(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_MODULATE);*/
				glBindTexture(GL_TEXTURE_2D, im.texture);
				shaderUniform1i(tex2Loc, 1);
			}
		}

		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();

		glTranslatef(x, y, 0.0f);
		glScalef(width, height, 1.0f);
		glColor4f(r, g, b, a);
		glBegin(GL_TRIANGLES);
		{
			for(i=1;i<p.numPoints;++i)
			{
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, p.xTex[0], p.yTex[0]);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 0.5f, 1.0f);
				glVertex2f(p.xPoints[0], p.yPoints[0]);

				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, p.xTex[i], p.yTex[i]);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 0.0f, 0.0f);
				glVertex2f(p.xPoints[i], p.yPoints[i]);

				if((i+1) < p.numPoints)
				{
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, p.xTex[i+1], p.yTex[i+1]);
					glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 1.0f, 0.0f);
					glVertex2f(p.xPoints[i+1], p.yPoints[i+1]);
				}
				else
				{
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, p.xTex[1], p.yTex[1]);
					glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 1.0f, 0.0f);
					glVertex2f(p.xPoints[1], p.yPoints[1]);
				}
			}
		}
		glEnd();
		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

		glPopMatrix();

		glActiveTextureARB(GL_TEXTURE1_ARB);
		glBindTexture(GL_TEXTURE_2D, 0);
		glDisable(GL_TEXTURE_2D);
		glActiveTextureARB(GL_TEXTURE0_ARB);
		glBindTexture(GL_TEXTURE_2D, 0);
	}
#ifdef DEBUG
	else
	{
		cout << "OpenGLVBODrawer::drawPoly(): Could not find image with id: ";
		cout << (unsigned int)textureId << endl;
	}
#endif
}

//------------------------------------------------------------------------------
void OpenGLVBODrawer::drawText(const string& text,
						    float x,
						    float y,
						    float height,
							float r,
							float g,
							float b,
							float a)
{
	int i;
	int tex;
	float xCount = x;
	float yCount = y;
	int tempSpace = glyphSpace * (height/glyphs[0].width);

	for(i=0;i<text.size();++i)
	{
		tex = getTexture(text[i]);

		if(tex != NumGlyphs)
		{
			glBindTexture(GL_TEXTURE_2D, glyphs[tex].texture);

			glColor4f(r, g, b, a);
			glBegin(GL_QUADS);
			{
				glTexCoord2f(0.0f, 0.0f);
				glVertex2f(xCount, yCount);

				glTexCoord2f(1.0f, 0.0f);
				glVertex2f(xCount+height, yCount);

				glTexCoord2f(1.0f, 1.0f);
				glVertex2f(xCount+height, yCount+height);

				glTexCoord2f(0.0f, 1.0f);
				glVertex2f(xCount, yCount+height);
			}
			glEnd();
			glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

			glBindTexture(GL_TEXTURE_2D, 0);
		}

		xCount += (height-tempSpace);
	}
}

//------------------------------------------------------------------------------
void OpenGLVBODrawer::drawText(const string& text,
							uint32_t font,
							float x,
							float y,
							float boxWidth,
							float scale,
							float r,
							float g,
							float b,
							float a)
{
	if(font == 0)
		drawText(text, x, y, 32.0f * scale, r, g, b, a);
	else
		//fonts[font-1]->drawAtlas();
		fonts[font-1]->draw(text, x, y, boxWidth, scale, r, g, b, a);
}

//------------------------------------------------------------------------------
void OpenGLVBODrawer::drawLine(float x1,
						    float y1,
						    float x2,
						    float y2,
						    float r,
						    float g,
						    float b,
						    float a)
{
	float x[4];
	float y[4];
	map<uint32_t, Image>::iterator it;
	const float dist = sqrtf(((x2-x1)*(x2-x1)) + ((y2-y1)*(y2-y1)));
	const float cX = (-(y2-y1)/dist)*4.0f;
	const float cY = ((x2-x1)/dist)*4.0f;

	//Bottom left point.
	x[0] = x1 + cX;
	y[0] = y1 + cY;
	//Bottom right point.
	x[1] = x2 + cX;
	y[1] = y2 + cY;
	//Top right point.
	x[2] = x2 - cX;
	y[2] = y2 - cY;
	//Top left point.
	x[3] = x1 - cX;
	y[3] = y1 - cY;

	it = images.find(lineImage);

	glBindTexture(GL_TEXTURE_2D, it->second.texture);
	glColor4f(r, g, b, a);
	glBegin(GL_QUADS);
	{
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 0.0f);
		glVertex2f(x[0], y[0]);

		glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 1.0f);
		glVertex2f(x[1], y[1]);

		glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 1.0f);
		glVertex2f(x[2], y[2]);

		glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 0.0f);
		glVertex2f(x[3], y[3]);
	}
	glEnd();

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glBindTexture(GL_TEXTURE_2D, 0);
}

//------------------------------------------------------------------------------
void OpenGLVBODrawer::fillRect(float x,
							float y,
							float w,
							float h,
							uint8_t r,
							uint8_t g,
							uint8_t b,
							uint8_t a)
{
	//Just in case.
	glBindTexture(GL_TEXTURE_2D, 0);

	glColor4ub(r, g, b, a);
	glBegin(GL_QUADS);
	{
		glVertex2f(x, y);
		glVertex2f(x+w, y);
		glVertex2f(x+w, y+h);
		glVertex2f(x, y+h);
	}
	glEnd();
}

//------------------------------------------------------------------------------
void OpenGLVBODrawer::fillGradient(float x,
							    float y,
							    float w,
							    float h,
							    uint8_t r1,
							    uint8_t g1,
							    uint8_t b1,
							    uint8_t a1,
							    uint8_t r2,
							    uint8_t g2,
							    uint8_t b2,
							    uint8_t a2)
{
	//Just in case.
	glBindTexture(GL_TEXTURE_2D, 0);

	glColor4ub(r1, g1, b1, a1);
	glBegin(GL_QUADS);
	{
		glVertex2f(x, y);
		glVertex2f(x+w, y);
		glColor4ub(r2, g2, b2, a2);
		glVertex2f(x+w, y+h);
		glVertex2f(x, y+h);
	}
	glEnd();
}

//------------------------------------------------------------------------------
void OpenGLVBODrawer::translate(float x, float y)
{
	glTranslatef(x, y, 0.0f);
}

//------------------------------------------------------------------------------
void OpenGLVBODrawer::setViewport(int16_t x, int16_t y, int16_t w, int16_t h)
{
	int16_t x2, y2, w2, h2;

	//Convert input coords to pixels.
	x2 = (int16_t)(((float)x/(float)getDrawWidth()) * viewW);
	y2 = (int16_t)(((float)y/(float)getDrawHeight()) * viewH);
	w2 = (int16_t)(((float)w/(float)getDrawWidth()) * viewW);
	h2 = (int16_t)(((float)h/(float)getDrawHeight()) * viewH);

	//Set the viewport.
	glViewport(viewX+x2, viewY+(screenHeight-(y2+h2)), w2, h2);

	//Now setup the projection matrix.
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	//So our co-ordinates for drawing are 0-w (x) and h-0 (y).
	gluOrtho2D(0.0f, (float)w, (float)h, 0.0f);

	glMatrixMode(GL_MODELVIEW);
}

//------------------------------------------------------------------------------
void OpenGLVBODrawer::resetViewport()
{
	glViewport((int)viewX, (int)viewY, (int)viewW, (int)viewH);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	//So our co-ordinates for drawing are 0-1024 (x) and 768-0 (y).
	gluOrtho2D(0.0f, 1024.0f, 768.0f, 0.0f);

	glMatrixMode(GL_MODELVIEW);
}

//------------------------------------------------------------------------------
bool OpenGLVBODrawer::is16Bit()
{
	bool retval = false;
	const SDL_VideoInfo *info = SDL_GetVideoInfo();

	if(info->vfmt->BitsPerPixel == 16)
		retval = true;

	return retval;
}

//------------------------------------------------------------------------------
void OpenGLVBODrawer::initFonts()
{
	int i;
	Image tempIm;
	uint8_t data[16*16*4];

	//Fill data.
	for(i=0;i<(16*16*4);++i)
	{
		data[i] = 255;
	}

	tempIm.width = 16;
	tempIm.height = 16;
	tempIm.texWidth = 1.0f;
	tempIm.texHeight = 1.0f;

	//Fill glyphs.
	for(i=0;i<NumGlyphs;++i)
	{
		glGenTextures(1, &(tempIm.texture));

		glBindTexture(GL_TEXTURE_2D, tempIm.texture);

		glTexImage2D(GL_TEXTURE_2D,
					 0,
					 GL_RGBA,
					 16,
					 16,
					 0,
					 GL_RGBA,
					 GL_UNSIGNED_BYTE,
					 data);

		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

		glBindTexture(GL_TEXTURE_2D, 0);

		glyphs[i] = tempIm;
	}
}

//------------------------------------------------------------------------------
bool OpenGLVBODrawer::isPowerOfTwo(uint32_t val)
{
	uint32_t tempint = 1;
	bool retval = false;

	//Surely there's a better way to do this?
	while(tempint < val)
		tempint *= 2;

	if(tempint == val)
		retval = true;

	return retval;
}

//------------------------------------------------------------------------------
uint32_t OpenGLVBODrawer::powerOfTwo(uint32_t val)
{
	uint32_t retval = 1;

	//Surely there's a better way to do this?
	while(retval < val)
		retval *= 2;

	return retval;
}

//------------------------------------------------------------------------------
void OpenGLVBODrawer::setupShaderFunctions()
{
	if(!glUniform1iARB || !GL_ARB_shader_objects)
	{
		cout << "Shader support: OpenGL shaders *NOT* available." << endl;
		shaderUniform1i = nullUniform1i;
		shaderUniform1f = nullUniform1f;
		shaderCreateShaderObject = nullCreateShaderObject;
		shaderShaderSource = nullShaderSource;
		shaderCompileShader = nullCompileShader;
		shaderCreateProgramObject = nullCreateProgramObject;
		shaderAttachObject = nullAttachObject;
		shaderLinkProgram = nullLinkProgram;
		shaderGetUniformLocation = nullGetUniformLocation;
		shaderUseProgramObject = nullUseProgramObject;

		shadersAreSupported = false;
	}
	else
	{
		cout << "Shader support: OpenGL shaders available." << endl;
		shaderUniform1i = glUniform1iARB;
		shaderUniform1f = glUniform1fARB;
		shaderCreateShaderObject = glCreateShaderObjectARB;
		shaderShaderSource = glShaderSourceARB;
		shaderCompileShader = glCompileShaderARB;
		shaderCreateProgramObject = glCreateProgramObjectARB;
		shaderAttachObject = glAttachObjectARB;
		shaderLinkProgram = glLinkProgramARB;
		shaderGetUniformLocation = glGetUniformLocationARB;
		shaderUseProgramObject = glUseProgramObjectARB;

		shadersAreSupported = true;
	}
}

//------------------------------------------------------------------------------
string OpenGLVBODrawer::loadShaderFile(const string& path)
{
	string retval;

	if(path.length() > 0)
	{
		ifstream phil(path.c_str());

		if(phil.is_open())
		{
			uint32_t length;
			char *buffer;

			phil.seekg(0, ios::end);
			length = phil.tellg();
			phil.seekg(0, ios::beg);

			buffer = new char[length+1];

			phil.read(buffer, length);
			if(phil.gcount() < (length+1))
				buffer[phil.gcount()] = '\0';
			else
				buffer[length] = '\0';
			//cout << buffer << endl;

			retval = buffer;

			delete [] buffer;
		//	phil >> retval.rdbuf();
		}
	}

	return retval;
}

//------------------------------------------------------------------------------
void OpenGLVBODrawer::printInfoLog(unsigned int obj)
{
    GLint infologLength = 0;
    GLsizei charsWritten  = 0;
    char *infoLog;

	if(!shadersAreSupported)
		return;
	
    glGetObjectParameterivARB(obj,
							  GL_OBJECT_INFO_LOG_LENGTH_ARB,
							  &infologLength);
	
    if(infologLength > 0)
    {
		infoLog = new char[infologLength];
		glGetInfoLogARB(obj, infologLength, &charsWritten, infoLog);
		cout << infoLog << endl;
		delete [] infoLog;
    }
}

//------------------------------------------------------------------------------
void GLAPIENTRY OpenGLVBODrawer::nullUniform1i(GLint, GLint)
{
	
}

//------------------------------------------------------------------------------
void GLAPIENTRY OpenGLVBODrawer::nullUniform1f(GLint, GLfloat)
{
	
}

//------------------------------------------------------------------------------
GLuint GLAPIENTRY OpenGLVBODrawer::nullCreateShaderObject(GLenum)
{
	return 0;
}

//------------------------------------------------------------------------------
void GLAPIENTRY OpenGLVBODrawer::nullShaderSource(GLuint, GLsizei, const GLchar **, const GLint *)
{
	
}

//------------------------------------------------------------------------------
void GLAPIENTRY OpenGLVBODrawer::nullCompileShader(GLuint)
{
	
}

//------------------------------------------------------------------------------
GLuint GLAPIENTRY OpenGLVBODrawer::nullCreateProgramObject(void)
{
	return 0;
}

//------------------------------------------------------------------------------
void GLAPIENTRY OpenGLVBODrawer::nullAttachObject(GLuint, GLuint)
{
	
}

//------------------------------------------------------------------------------
void GLAPIENTRY OpenGLVBODrawer::nullLinkProgram(GLuint)
{

}

//------------------------------------------------------------------------------
GLint GLAPIENTRY OpenGLVBODrawer::nullGetUniformLocation(GLuint, const GLchar *)
{
	return 0;
}

//------------------------------------------------------------------------------
void GLAPIENTRY OpenGLVBODrawer::nullUseProgramObject(GLuint)
{

}

//------------------------------------------------------------------------------
static const unsigned char temp1[] = {137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,8,0,0,0,8,8,6,0,0,0,196,15,190,139,0,0,0,7,116,73,77,
  69,7,217,11,5,18,24,55,156,239,108,44,0,0,0,9,112,72,89,115,0,0,13,172,0,0,13,172,1,239,6,198,71,0,0,0,4,103,65,77,
  65,0,0,177,143,11,252,97,5,0,0,0,31,73,68,65,84,120,218,99,248,255,255,63,19,16,43,2,241,129,255,8,112,0,42,198,196,196,64,0,
  12,15,5,0,208,222,27,187,199,235,156,253,0,0,0,0,73,69,78,68,174,66,96,130,0,0};
const char* OpenGLVBODrawer::line_png = (const char*)temp1;
